@@ -1,18 +1,33 @@
 """
+app.py
+
 PyGPSClient - Main tkinter application class.
 
+- Loads configuration from json file (if available)
+- Instantiates all frames, widgets, and protocol handlers.
+- Starts and stops threaded dialog and protocol handler processes.
+- Maintains current serial and RTK connection status.
+- Reacts to various message events, processes navigation data
+  placed on input message queue by serial, socket or file stream reader
+  and assigns to appropriate NMEA, UBX or RTCM protocol handler.
+- Maintains central dictionary of current key navigation data as
+  `gnss_status`, for use by user-selectable widgets.
+
 Created on 12 Sep 2020
 
 :author: semuadmin
 :copyright: SEMU Consulting © 2020
 :license: BSD 3-Clause
 """
+# pylint: disable=too-many-ancestors, no-member
 
 from datetime import datetime, timedelta
-from os import getenv
+from os import getenv, path
+from pathlib import Path
 from queue import Empty, Queue
+from socket import AF_INET, AF_INET6
 from threading import Thread
-from tkinter import E, Frame, N, PhotoImage, S, Tk, Toplevel, W, font
+from tkinter import E, Frame, N, PhotoImage, S, TclError, Tk, Toplevel, W, font
 
 from pygnssutils import GNSSMQTTClient, GNSSNTRIPClient
 from pygnssutils.socket_server import ClientHandler, SocketServer
@@ -20,81 +35,74 @@ from pyubx2 import NMEA_PROTOCOL, RTCM3_PROTOCOL, UBX_PROTOCOL, protocol
 from serial import SerialException, SerialTimeoutException
 
 from pygpsclient._version import __version__ as VERSION
-from pygpsclient.about_dialog import AboutDialog
-from pygpsclient.banner_frame import BannerFrame
-from pygpsclient.console_frame import ConsoleFrame
+from pygpsclient.dialog_state import dialog_state
 from pygpsclient.file_handler import FileHandler
 from pygpsclient.globals import (
     BADCOL,
+    CFG,
     CHECK_FOR_UPDATES,
+    CLASS,
     CONFIGFILE,
     CONNECTED,
     DISCONNECTED,
     DLG,
-    DLGTABOUT,
-    DLGTGPX,
     DLGTNTRIP,
-    DLGTSPARTN,
-    DLGTUBX,
-    FRM,
+    FRAME,
     GNSS_EOF_EVENT,
+    GNSS_ERR_EVENT,
     GNSS_EVENT,
     GUI_UPDATE_INTERVAL,
     ICON_APP,
-    MAP_UPDATE_INTERVAL,
     NOPORTS,
     NTRIP_EVENT,
     OKCOL,
-    SOCKSERVER_HOST,
     SOCKSERVER_MAX_CLIENTS,
     SPARTN_EVENT,
+    SPARTN_OUTPORT,
+    SPARTN_PPSERVER,
     THD,
 )
 from pygpsclient.gnss_status import GNSSStatus
-from pygpsclient.gpx_dialog import GPXViewerDialog
-from pygpsclient.graphview_frame import GraphviewFrame
 from pygpsclient.helpers import check_latest
-from pygpsclient.map_frame import MapviewFrame
 from pygpsclient.menu_bar import MenuBar
 from pygpsclient.nmea_handler import NMEAHandler
-from pygpsclient.ntrip_client_dialog import NTRIPConfigDialog
 from pygpsclient.rtcm3_handler import RTCM3Handler
-from pygpsclient.scatter_frame import ScatterViewFrame
-from pygpsclient.settings_frame import SettingsFrame
-from pygpsclient.skyview_frame import SkyviewFrame
-from pygpsclient.spartn_dialog import SPARTNConfigDialog
-from pygpsclient.spectrum_frame import SpectrumviewFrame
-from pygpsclient.status_frame import StatusFrame
 from pygpsclient.stream_handler import StreamHandler
 from pygpsclient.strings import (
     CONFIGERR,
+    DLGSTOPRTK,
+    ENDOFFILE,
     INTROTXTNOPORTS,
     LOADCONFIGBAD,
+    LOADCONFIGNONE,
     LOADCONFIGOK,
     NOTCONN,
+    NOWDGSWARN,
     SAVECONFIGBAD,
     SAVECONFIGOK,
     TITLE,
     VERCHECK,
 )
-from pygpsclient.ubx_config_dialog import UBXConfigDialog
 from pygpsclient.ubx_handler import UBXHandler
-from pygpsclient.widgets import (
+from pygpsclient.widget_state import (
+    COLSPAN,
+    DEFAULT,
     HIDE,
     MAXCOLSPAN,
     MAXROWSPAN,
+    MENU,
+    ROWSPAN,
     SHOW,
+    STICKY,
+    VISIBLE,
     WDGBANNER,
     WDGSETTINGS,
-    WDGSPECTRUM,
     WDGSTATUS,
-    widget_grid,
+    widget_state,
 )
 
-SPARTN_PROTOCOL = 9
 
-
-class App(Frame):  # pylint: disable=too-many-ancestors
+class App(Frame):
     """
     Main PyGPSClient GUI Application Class.
     """
@@ -112,13 +120,22 @@ class App(Frame):  # pylint: disable=too-many-ancestors
 
         # user-defined serial port can be passed as environment variable
         # or command line keyword argument
-        configfile = kwargs.pop("config", CONFIGFILE)
-        self._user_port = kwargs.pop("userport", getenv("PYGPSCLIENT_USERPORT", ""))
-        self._spartn_user_port = kwargs.pop(
+        self._configfile = kwargs.pop("config", CONFIGFILE)
+        self.user_port = kwargs.pop("userport", getenv("PYGPSCLIENT_USERPORT", ""))
+        self.spartn_user_port = kwargs.pop(
             "spartnport", getenv("PYGPSCLIENT_SPARTNPORT", "")
         )
-        self._mqapikey = kwargs.pop("mqapikey", getenv("MQAPIKEY", ""))
-        self._mqttclientid = kwargs.pop("mqttclientid", getenv("MQTTCLIENTID", ""))
+        self.mqapikey = kwargs.pop("mqapikey", getenv("MQAPIKEY", ""))
+        self.mqttclientid = kwargs.pop("mqttclientid", getenv("MQTTCLIENTID", ""))
+        self.mqttclientregion = kwargs.pop(
+            "mqttclientregion", getenv("MQTTCLIENTREGION", "eu")
+        )
+        self.ntripcaster_user = kwargs.pop(
+            "ntripuser", getenv("PYGPSCLIENT_USER", "anon")
+        )
+        self.ntripcaster_password = kwargs.pop(
+            "ntrippassword", getenv("PYGPSCLIENT_USER", "password")
+        )
 
         Frame.__init__(self, self.__master, *args, **kwargs)
 
@@ -127,8 +144,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         self.__master.iconphoto(True, PhotoImage(file=ICON_APP))
         self.gnss_status = GNSSStatus()  # holds latest GNSS readings
         self._last_gui_update = datetime.now()
-        # dict containing widget grid positions
-        self._widget_grid = widget_grid
+        self._nowidgets = True
 
         # Instantiate protocol handler classes
         self.gnss_inqueue = Queue()  # messages from GNSS receiver
@@ -146,59 +162,55 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         self.rtcm_handler = RTCM3Handler(self)
         self.ntrip_handler = GNSSNTRIPClient(self, verbosity=0)
         self.spartn_handler = GNSSMQTTClient(self, verbosity=0)
-        self.dlg_threads = {}
-        self.config = {}
+        self.saved_config = {}
         self._conn_status = DISCONNECTED
         self._rtk_conn_status = DISCONNECTED
-        self._map_update_interval = MAP_UPDATE_INTERVAL
         self._socket_thread = None
         self._socket_server = None
         self._colcount = 0
         self._rowcount = 0
 
-        # FOLLOWING FILE LOADS ARE DEPRECATED - USE CONFIG FILE INSTEAD
-        # (ANY CONFIG FILE SETTINGS WILL OVERRIDE THESE)
-        # Load MapQuest web map api key if not already defined
-        if self._mqapikey == "":
-            self._mqapikey = self.file_handler.load_mqapikey()
-        # Load console color tags from file
-        self._colortags = self.file_handler.load_colortags()
-        # Load user-defined UBX command presets from file
-        self._ubxpresets = self.file_handler.load_ubx_presets()
-
         # Load configuration from file if it exists
-        config_loaded = False
-        _, self.config = self.file_handler.load_config(configfile)
-        if isinstance(self.config, dict):  # load succeeded
-            config_loaded = True
-            self.app_config = self.config  # do this before initialising widgets
+        self._colortags = []
+        self._ubxpresets = []
+        (_, config, configerr) = self.file_handler.load_config(self._configfile)
+        if configerr == "":  # load succeeded
+            self.saved_config = config
+            # update configs needed to instantiate widgets and protocol handlers
+            self.update_widgets()
+        else:  # load failed - invalid json or attribute types
+            self.saved_config = {}
+
+        # update NTRIP and SPARTN client handlers with initial config
+        self.update_NTRIP_handler()
+        self.update_SPARTN_handler()
 
         self._body()
         self._do_layout()
-        self._init_dialogs()
         self._attach_events()
 
-        # Initialise widgets
-        if config_loaded:
-            self.widget_config = self.config
-            self.frm_settings.config = self.config
-            self.frm_status.set_status(f"{LOADCONFIGOK} {configfile}", OKCOL)
-        else:
-            # self.config is str containing error msg
-            if "No such file or directory" in self.config:
-                self.frm_status.set_status(f"Configuration file not found {configfile}")
+        # display config load status once status frame has been instantiated
+        if configerr == "":
+            if self._nowidgets:  # if all widgets have been disabled in config
+                self.set_status(NOWDGSWARN.format(self._configfile), BADCOL)
             else:
-                self.frm_status.set_status(
-                    f"{LOADCONFIGBAD} {configfile} {self.config}"
+                self.set_status(LOADCONFIGOK.format(self._configfile), OKCOL)
+        else:
+            if "No such file or directory" in configerr:
+                self.set_status(LOADCONFIGNONE.format(self._configfile), BADCOL)
+            else:
+                self.set_status(
+                    LOADCONFIGBAD.format(self._configfile, configerr), BADCOL
                 )
-            self.config = {}
-        self.frm_satview.init_sats()
-        self.frm_graphview.init_graph()
-        self.frm_spectrumview.init_graph()
-        self.frm_scatterview.init_graph()
-        self.frm_banner.update_conn_status(DISCONNECTED)
 
-        if self.frm_settings.serial_settings.status == NOPORTS:
+        # initialise widgets
+        for value in widget_state.values():
+            frm = getattr(self, value[FRAME])
+            if hasattr(frm, "init_frame"):
+                frm.init_frame()
+
+        self.frm_banner.update_conn_status(DISCONNECTED)
+        if self.frm_settings.frm_serial.status == NOPORTS:
             self.set_status(INTROTXTNOPORTS, BADCOL)
 
         # Check for more recent version (if enabled)
@@ -211,20 +223,15 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         """
 
         self._set_default_fonts()
-
         self.menu = MenuBar(self)
-        self.frm_status = StatusFrame(self, borderwidth=2, relief="groove")
-        self.frm_banner = BannerFrame(self, borderwidth=2, relief="groove")
-        self.frm_settings = SettingsFrame(self, borderwidth=2, relief="groove")
-        self.frm_console = ConsoleFrame(self, borderwidth=2, relief="groove")
-        self.frm_mapview = MapviewFrame(self, borderwidth=2, relief="groove")
-        self.frm_satview = SkyviewFrame(self, borderwidth=2, relief="groove")
-        self.frm_graphview = GraphviewFrame(self, borderwidth=2, relief="groove")
-        self.frm_spectrumview = SpectrumviewFrame(self, borderwidth=2, relief="groove")
-        self.frm_scatterview = ScatterViewFrame(self, borderwidth=2, relief="groove")
-
         self.__master.config(menu=self.menu)
 
+        # dynamically instantiate widgets defined in widgets_grid
+        for value in widget_state.values():
+            _ = setattr(
+                self, value[FRAME], value[CLASS](self, borderwidth=2, relief="groove")
+            )
+
     def _do_layout(self):
         """
         Arrange widgets in main application frame, and set
@@ -233,7 +240,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
 
         col = mcol = 0
         row = mrow = 1
-        for i, nam in enumerate(self._widget_grid):
+        for i, nam in enumerate(widget_state):
             if i > 2:  # only position dynamic widgets
                 col, row = self._grid_widget(nam, col, row)
             mcol = max(col, mcol)
@@ -263,41 +270,43 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         :rtype: tuple
         """
 
-        wdg = self._widget_grid[nam]
-        if wdg["visible"]:
-            colspan = wdg.get("colspan", colspan)
-            rowspan = wdg.get("rowspan", rowspan)
+        wdg = widget_state[nam]
+        frm = getattr(self, wdg[FRAME])
+        if wdg[VISIBLE]:
+            colspan = wdg.get(COLSPAN, colspan)
+            rowspan = wdg.get(ROWSPAN, rowspan)
             if col >= MAXCOLSPAN and nam != WDGSETTINGS:
                 col = 0
                 row += rowspan
             # keep track of cumulative cols & rows
             ccol = wdg.get("col", col)
             crow = wdg.get("row", row)
-            getattr(self, wdg["frm"]).grid(
+            frm.grid(
                 column=ccol,
                 row=crow,
                 columnspan=colspan,
                 rowspan=rowspan,
                 padx=2,
                 pady=2,
-                sticky=wdg.get("sticky", (N, S, W, E)),
+                sticky=wdg.get(STICKY, (N, S, W, E)),
             )
             col += colspan
             lbl = HIDE
         else:
-            getattr(self, wdg["frm"]).grid_forget()
+            frm.grid_forget()
             lbl = SHOW
 
         # update menu label (show/hide)
-        if wdg["menu"] is not None:
-            self.menu.view_menu.entryconfig(wdg["menu"], label=f"{lbl} {nam}")
-
-        if nam == WDGSPECTRUM:
-            # enable MON-SPAN messages if spectrum widget is visible
-            self.frm_spectrumview.enable_MONSPAN(wdg["visible"])
+        if wdg[MENU] is not None:
+            self.menu.view_menu.entryconfig(wdg[MENU], label=f"{lbl} {nam}")
 
         # force widget to rescale
-        getattr(self, wdg["frm"]).event_generate("<Configure>")
+        frm.event_generate("<Configure>")
+
+        # enable or disable any UBX messages required by widget
+        if hasattr(frm, "enable_messages"):
+            frm.enable_messages(wdg[VISIBLE])
+
         return col, row
 
     def toggle_widget(self, widget: str):
@@ -307,7 +316,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         :param str widget: widget name
         """
 
-        self._widget_grid[widget]["visible"] = not self._widget_grid[widget]["visible"]
+        widget_state[widget][VISIBLE] = not widget_state[widget][VISIBLE]
         self._do_layout()
 
     def reset_widgets(self):
@@ -315,23 +324,16 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         Reset widgets to default layout.
         """
 
-        for _, wdg in self._widget_grid.items():
-            wdg["visible"] = wdg["default"]
+        for _, wdg in widget_state.items():
+            wdg[VISIBLE] = wdg[DEFAULT]
         self._do_layout()
 
-    def _init_dialogs(self):
+    def reset_gnssstatus(self):
         """
-        Initialise dictionary of dialog statuses.
+        Reset gnss_status dict e.g. after reconnecting.
         """
 
-        self.dlg_threads = {
-            DLGTABOUT: {FRM: AboutDialog, THD: None, DLG: None},
-            DLGTUBX: {FRM: UBXConfigDialog, THD: None, DLG: None},
-            DLGTNTRIP: {FRM: NTRIPConfigDialog, THD: None, DLG: None},
-            DLGTSPARTN: {FRM: SPARTNConfigDialog, THD: None, DLG: None},
-            DLGTGPX: {FRM: GPXViewerDialog, THD: None, DLG: None},
-            # add any new dialogs here
-        }
+        self.gnss_status = GNSSStatus()
 
     def _attach_events(self):
         """
@@ -339,7 +341,8 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         """
 
         self.__master.bind(GNSS_EVENT, self.on_gnss_read)
-        self.__master.bind(GNSS_EOF_EVENT, self.stream_handler.on_eof)
+        self.__master.bind(GNSS_EOF_EVENT, self.on_gnss_eof)
+        self.__master.bind(GNSS_ERR_EVENT, self.on_stream_error)
         self.__master.bind(NTRIP_EVENT, self.on_ntrip_read)
         self.__master.bind(SPARTN_EVENT, self.on_spartn_read)
         self.__master.bind_all("<Control-q>", self.on_exit)
@@ -356,7 +359,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         self.font_md2 = font.Font(size=14)
         self.font_lg = font.Font(size=18)
 
-    def set_connection(self, message, color="green"):
+    def set_connection(self, message, color=OKCOL):
         """
         Sets connection description in status bar.
 
@@ -367,7 +370,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
 
         self.frm_status.set_connection(message, color=color)
 
-    def set_status(self, message, color="green"):
+    def set_status(self, message, color=OKCOL):
         """
         Sets text of status bar.
 
@@ -392,36 +395,160 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         Load configuration file menu option.
         """
 
-        filename, self.config = self.file_handler.load_config(None)
-        if self.config is None:
-            return  # user cancelled
-        if isinstance(self.config, str):  # load failed
-            self.set_status(f"{LOADCONFIGBAD} {self.config}", BADCOL)
+        # Warn if Streaming, NTRIP or SPARTN clients are running
+        if self.conn_status == DISCONNECTED and self.rtk_conn_status == DISCONNECTED:
+            self.set_status("", OKCOL)
         else:
-            self.set_status(f"{LOADCONFIGOK} {filename}", OKCOL)
-            self.frm_settings.config = self.config
-            self.widget_config = self.config
-            self.app_config = self.config
+            self.set_status(DLGSTOPRTK, BADCOL)
+            return
+
+        (filename, config, configerr) = self.file_handler.load_config(None)
+        if configerr == "":  # load succeeded
+            self.saved_config = config
+            self.update_widgets()
+            self.update_NTRIP_handler()
+            self.update_SPARTN_handler()
+            for frm in (
+                self.frm_settings,
+                self.frm_settings.frm_serial,
+                self.frm_settings.frm_socketclient,
+                self.frm_settings.frm_socketserver,
+            ):
+                frm.reset()
+            self._do_layout()
+            if self._nowidgets:
+                self.set_status(NOWDGSWARN.format(filename), BADCOL)
+            else:
+                self.set_status(LOADCONFIGOK.format(filename), OKCOL)
+        elif configerr == "cancelled":  # user cancelled
+            return
+        else:  # config error
+            self.set_status(LOADCONFIGBAD.format(filename), BADCOL)
 
     def save_config(self):
         """
         Save configuration file menu option.
+
+        Configuration is held in two places:
+        - app.config - widget visibility and CLI parameters
+        - frm_settings.config - current frame and protocol handler config
         """
 
-        # combine the various config sections into one dict
-        self.config = {
-            **self.frm_settings.config,
+        self.saved_config = {
             **self.widget_config,
-            **self.app_config,
+            **self.frm_settings.config,
         }
-        rcd = self.file_handler.save_config(self.config, None)
-        if rcd is None:
-            return
-        if isinstance(rcd, str):  # save failed
-            self.set_status(f"{SAVECONFIGBAD} {self.config}", BADCOL)
-            self.config = None
-        else:
+        err = self.file_handler.save_config(self.saved_config, None)
+        if err == "":
             self.set_status(SAVECONFIGOK, OKCOL)
+        else:  # save failed
+            self.set_status(SAVECONFIGBAD.format(err), BADCOL)
+
+    def update_widgets(self):
+        """
+        Update widget configuration (widget_state).
+
+        If no widgets are set as "visible" in the config file, enable
+        the status widget anyway to display a warning to this effect.
+        """
+
+        try:
+            self._nowidgets = True
+            for key, vals in widget_state.items():
+                vis = self.saved_config.get(key, False)
+                vals[VISIBLE] = vis
+                if vis:
+                    self._nowidgets = False
+            if self._nowidgets:
+                widget_state["Status"][VISIBLE] = "true"
+        except KeyError as err:
+            self.set_status(f"{CONFIGERR} - {err}", BADCOL)
+
+    def update_NTRIP_handler(self):
+        """
+        Initial configuration of NTRIP handler (pygnssutils.GNSSNTRIPClient).
+        """
+
+        try:
+            ntripsettings = {}
+            ntripsettings["server"] = self.saved_config.get("ntripclientserver_s", "")
+            ntripsettings["port"] = self.saved_config.get("ntripclientport_n", 2101)
+            ntripsettings["ipprot"] = (
+                AF_INET6
+                if self.saved_config.get("ntripclientprotocol_s") == "IPv6"
+                else AF_INET
+            )
+            ntripsettings["flowinfo"] = self.saved_config.get(
+                "ntripclientflowinfo_n", 0
+            )
+            ntripsettings["scopeid"] = self.saved_config.get("ntripclientscopeid_n", 0)
+            ntripsettings["mountpoint"] = self.saved_config.get(
+                "ntripclientmountpoint_s", ""
+            )
+            ntripsettings["sourcetable"] = []  # this is generated by the NTRIP caster
+            ntripsettings["version"] = self.saved_config.get(
+                "ntripclientversion_s", "2.0"
+            )
+            ntripsettings["ntripuser"] = self.saved_config.get(
+                "ntripclientuser_s", "anon"
+            )
+            ntripsettings["ntrippassword"] = self.saved_config.get(
+                "ntripclientpassword_s", "password"
+            )
+            ntripsettings["ggainterval"] = self.saved_config.get(
+                "ntripclientggainterval_n", -1
+            )
+            ntripsettings["ggamode"] = self.saved_config.get(
+                "ntripclientggamode_b", 0
+            )  # GGALIVE
+            ntripsettings["reflat"] = self.saved_config.get("ntripclientreflat_f", 0.0)
+            ntripsettings["reflon"] = self.saved_config.get("ntripclientreflon_f", 0.0)
+            ntripsettings["refalt"] = self.saved_config.get("ntripclientrefalt_f", 0.0)
+            ntripsettings["refsep"] = self.saved_config.get("ntripclientrefsep_f", 0.0)
+            self.ntrip_handler.settings = ntripsettings
+
+        except (KeyError, ValueError, TypeError, TclError) as err:
+            self.set_status(f"Error processing config data: {err}", BADCOL)
+
+    def update_SPARTN_handler(self):
+        """
+        Initial configuration of SPARTN handler (pygnssutils.GNSSMQTTClient).
+        """
+
+        try:
+            spartnsettings = {}
+            spartnsettings["server"] = self.saved_config.get(
+                "mqttclientserver_s", SPARTN_PPSERVER
+            )
+            spartnsettings["port"] = self.saved_config.get(
+                "mqttclientport_n", SPARTN_OUTPORT
+            )
+            spartnsettings["clientid"] = self.saved_config.get(
+                "mqttclientid_s", self.mqttclientid
+            )
+            spartnsettings["region"] = self.saved_config.get(
+                "mgttclientregion_s", self.mqttclientregion
+            )
+            spartnsettings["topic_ip"] = self.saved_config.get("mgttclienttopicip_b", 1)
+            spartnsettings["topic_mga"] = self.saved_config.get(
+                "mgttclienttopicmga_b", 1
+            )
+            spartnsettings["topic_key"] = self.saved_config.get(
+                "mgttclienttopickey_b", 1
+            )
+            spartnsettings["tlscrt"] = self.saved_config.get(
+                "mgttclienttlscrt_s",
+                path.join(Path.home(), f"device-{self.mqttclientid}-pp-cert.crt"),
+            )
+            spartnsettings["tlskey"] = self.saved_config.get(
+                "mgttclienttlskey_s",
+                path.join(Path.home(), f"device-{self.mqttclientid}-pp-key.pem"),
+            )
+            spartnsettings["output"] = self.spartn_inqueue
+            self.spartn_handler.settings = spartnsettings
+
+        except (KeyError, ValueError, TypeError, TclError) as err:
+            self.set_status(f"Error processing config data: {err}", BADCOL)
 
     def start_dialog(self, dlg: str):
         """
@@ -430,11 +557,11 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         :param str dlg: name of dialog
         """
 
-        if self.dlg_threads[dlg][THD] is None:
-            self.dlg_threads[dlg][THD] = Thread(
+        if dialog_state[dlg][THD] is None:
+            dialog_state[dlg][THD] = Thread(
                 target=self._dialog_thread, args=(dlg,), daemon=False
             )
-            self.dlg_threads[dlg][THD].start()
+            dialog_state[dlg][THD].start()
 
     def _dialog_thread(self, dlg: str):
         """
@@ -445,8 +572,9 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         :param str dlg: name of dialog
         """
 
-        frm = self.dlg_threads[dlg][FRM]
-        self.dlg_threads[dlg][DLG] = frm(self)
+        config = self.saved_config if dialog_state[dlg][CFG] else {}
+        cls = dialog_state[dlg][CLASS]
+        dialog_state[dlg][DLG] = cls(self, saved_config=config)
 
     def stop_dialog(self, dlg: str):
         """
@@ -455,8 +583,8 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         :param str dlg: name of dialog
         """
 
-        self.dlg_threads[dlg][THD] = None
-        self.dlg_threads[dlg][DLG] = None
+        dialog_state[dlg][THD] = None
+        dialog_state[dlg][DLG] = None
 
     def dialog(self, dlg: str) -> Toplevel:
         """
@@ -467,7 +595,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         :rtype: Toplevel
         """
 
-        return self.dlg_threads[dlg][DLG]
+        return dialog_state[dlg][DLG]
 
     def start_sockserver_thread(self):
         """
@@ -475,14 +603,19 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         """
 
         settings = self.frm_settings.config
-        port = int(settings["sockport"])
-        ntripmode = settings["sockmode"]
+        host = settings["sockhost_s"]
+        port = int(settings["sockport_n"])
+        ntripmode = int(settings["sockmode_b"])
+        ntripuser = settings["ntripcasteruser_s"]
+        ntrippassword = settings["ntripcasterpassword_s"]
         self._socket_thread = Thread(
             target=self._sockserver_thread,
             args=(
                 ntripmode,
-                SOCKSERVER_HOST,
+                host,
                 port,
+                ntripuser,
+                ntrippassword,
                 SOCKSERVER_MAX_CLIENTS,
                 self.socket_outqueue,
             ),
@@ -501,7 +634,14 @@ class App(Frame):  # pylint: disable=too-many-ancestors
             self._socket_server.shutdown()
 
     def _sockserver_thread(
-        self, ntripmode: int, host: str, port: int, maxclients: int, socketqueue: Queue
+        self,
+        ntripmode: int,
+        host: str,
+        port: int,
+        ntripuser: str,
+        ntrippassword: str,
+        maxclients: int,
+        socketqueue: Queue,
     ):
         """
         THREADED
@@ -516,7 +656,14 @@ class App(Frame):  # pylint: disable=too-many-ancestors
 
         try:
             with SocketServer(
-                self, ntripmode, maxclients, socketqueue, (host, port), ClientHandler
+                self,
+                ntripmode,
+                maxclients,
+                socketqueue,
+                (host, port),
+                ClientHandler,
+                ntripuser=ntripuser,
+                ntrippassword=ntrippassword,
             ) as self._socket_server:
                 self._socket_server.serve_forever()
         except OSError as err:
@@ -529,7 +676,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         :param int clients: no of connected clients
         """
 
-        self.frm_settings.clients = clients
+        self.frm_settings.frm_socketserver.clients = clients
 
     def on_exit(self, *args, **kwargs):  # pylint: disable=unused-argument
         """
@@ -545,7 +692,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
     def on_gnss_read(self, event):  # pylint: disable=unused-argument
         """
         EVENT TRIGGERED
-        Action on <<gnss_read>> event - read data from GNSS queue.
+        Action on <<gnss_read>> event - data available on GNSS queue.
 
         :param event event: read event
         """
@@ -554,14 +701,31 @@ class App(Frame):  # pylint: disable=too-many-ancestors
             raw_data, parsed_data = self.gnss_inqueue.get(False)
             if raw_data is not None and parsed_data is not None:
                 self.process_data(raw_data, parsed_data)
+            # if socket server is running, output raw data to socket
+            if self.frm_settings.frm_socketserver.socketserving:
+                self.socket_outqueue.put(raw_data)
             self.gnss_inqueue.task_done()
         except Empty:
             pass
 
+    def on_gnss_eof(self, event):  # pylint: disable=unused-argument
+        """
+        EVENT TRIGGERED
+        Action on <<sgnss_eof>> event - end of file.
+
+        :param event event: <<gnss_eof>> event
+        """
+
+        self.frm_settings.frm_socketserver.socketserving = (
+            False  # turn off socket server
+        )
+        self.conn_status = DISCONNECTED
+        self.set_status(ENDOFFILE)
+
     def on_ntrip_read(self, event):  # pylint: disable=unused-argument
         """
         EVENT TRIGGERED
-        Action on <<ntrip_read>> event - read data from NTRIP queue.
+        Action on <<ntrip_read>> event - data available on NTRIP queue.
 
         :param event event: read event
         """
@@ -584,7 +748,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
     def on_spartn_read(self, event):  # pylint: disable=unused-argument
         """
         EVENT TRIGGERED
-        Action on <<spartn_read>> event - read data from SPARTN queue.
+        Action on <<spartn_read>> event - data available on SPARTN queue.
 
         :param event event: read event
         """
@@ -602,6 +766,16 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         except (SerialException, SerialTimeoutException) as err:
             self.set_status(f"Error sending to device {err}", BADCOL)
 
+    def on_stream_error(self, event):  # pylint: disable=unused-argument
+        """
+        EVENT TRIGGERED
+        Action on "<<gnss_error>>" event - connection streaming error.
+
+        :param event event: <<gnss_error>> event
+        """
+
+        self.conn_status = DISCONNECTED
+
     def update_ntrip_status(self, status: bool, msgt: tuple = None):
         """
         Update NTRIP configuration dialog connection status.
@@ -640,7 +814,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
 
         settings = self.frm_settings.config
 
-        protfilter = settings["protocol"]
+        protfilter = settings["protocol_n"]
         msgprot = protocol(raw_data)
         if isinstance(parsed_data, str):  # error message rather than parsed data
             marker = "WARNING  "
@@ -654,7 +828,7 @@ class App(Frame):  # pylint: disable=too-many-ancestors
             self.rtcm_handler.process_data(raw_data, parsed_data)
 
         # update console if visible
-        if self._widget_grid["Console"]["visible"]:
+        if widget_state["Console"][VISIBLE]:
             if msgprot == 0 or msgprot & protfilter:
                 self.frm_console.update_console(raw_data, parsed_data, marker)
 
@@ -663,18 +837,18 @@ class App(Frame):  # pylint: disable=too-many-ancestors
             seconds=GUI_UPDATE_INTERVAL
         ):
             self.frm_banner.update_frame()
-            for _, widget in self._widget_grid.items():
+            for _, widget in widget_state.items():
                 frm = getattr(self, widget["frm"])
-                if hasattr(frm, "update_frame") and widget["visible"]:
+                if hasattr(frm, "update_frame") and widget[VISIBLE]:
                     frm.update_frame()
             self._last_gui_update = datetime.now()
 
         # update GPX track file if enabled
-        if settings["recordtrack"]:
+        if settings["recordtrack_b"]:
             self.file_handler.update_gpx_track()
 
         # update log file if enabled
-        if settings["datalog"]:
+        if settings["datalog_b"]:
             self.file_handler.write_logfile(raw_data, parsed_data)
 
     def _check_update(self):
@@ -687,72 +861,37 @@ class App(Frame):  # pylint: disable=too-many-ancestors
             self.set_status(VERCHECK.format(latest), BADCOL)
 
     @property
-    def app_config(self) -> dict:
+    def widget_config(self) -> dict:
         """
-        Getter for application configuration.
-
-        This contains user-defined ports, various API keys
-        and colortagging values.
+        Getter for widget configuration.
 
         :return: configuration
         :rtype: dict
         """
 
-        config = {
-            "mapupdateinterval": self._map_update_interval,
-            "userport": self._user_port,
-            "spartnport": self._spartn_user_port,
-            "mqapikey": self._mqapikey,
-            "mqttclientid": self._mqttclientid,
-            "colortags": self._colortags,
-            "ubxpresets": self._ubxpresets,
-        }
-        return config
-
-    @app_config.setter
-    def app_config(self, config):
-        """
-        Setter for application configuration.
-
-        :param dict config: configuration as dict
-        """
-
-        self._map_update_interval = config.get("mapupdateinterval", MAP_UPDATE_INTERVAL)
-        self._user_port = config.get("userport", self._user_port)
-        self._spartn_user_port = config.get("spartnport", self._spartn_user_port)
-        self._mqapikey = config.get("mqapikey", self._mqapikey)
-        self._mqttclientid = config.get("mqttclientid", self._mqttclientid)
-        self._colortags = config.get("colortags", self._colortags)
-        self._ubxpresets = config.get("ubxpresets", self._ubxpresets)
+        return {key: vals[VISIBLE] for key, vals in widget_state.items()}
 
     @property
-    def widget_config(self) -> dict:
+    def widgets(self) -> dict:
         """
-        Getter for widget configuration.
+        Getter for widget state.
 
-        This contains the current status (visibility) of the various widgets.
-
-        :return: widget configuration as dict
+        :return: widget state
         :rtype: dict
         """
 
-        return {key: vals["visible"] for key, vals in self._widget_grid.items()}
+        return widget_state
 
-    @widget_config.setter
-    def widget_config(self, config):
+    @property
+    def dialogs(self) -> dict:
         """
-        Setter for widget configuration.
+        Getter for dialog state.
 
-        :param dict config: configuration as dict
+        :return: dialog state
+        :rtype: dict
         """
 
-        try:
-            for key, vals in self._widget_grid.items():
-                vals["visible"] = config[key]
-        except KeyError as err:
-            self.set_status(f"{CONFIGERR} - {err}", BADCOL)
-
-        self._do_layout()
+        return dialog_state
 
     @property
     def appmaster(self) -> Tk:
@@ -789,7 +928,6 @@ class App(Frame):  # pylint: disable=too-many-ancestors
         self.frm_settings.enable_controls(status)
         if status == DISCONNECTED:
             self.set_connection(NOTCONN)
-            self.set_status("")
 
     @property
     def rtk_conn_status(self) -> int:
@@ -812,3 +950,15 @@ class App(Frame):  # pylint: disable=too-many-ancestors
 
         self._rtk_conn_status = status
         self.frm_banner.update_rtk_status(status)
+
+    def svin_countdown(self, dur: int, valid: bool, active: bool):
+        """
+        Countdown survey-in duration for NTRIP caster mode.
+
+        :param int dur: elapsed time
+        :param bool valid: valid flag
+        :param bool active: active flag
+        """
+
+        if self.frm_settings.frm_socketserver is not None:
+            self.frm_settings.frm_socketserver.svin_countdown(dur, valid, active)
