@@ -1,4 +1,6 @@
 """
+globals.py
+
 PyGPSClient Globals
 
 Collection of global constants
@@ -20,72 +22,96 @@ from pathlib import Path
 from pyubx2 import GET, POLL, SET
 
 Point = namedtuple("Point", ["lat", "lon"])
-
 HOME = Path.home()
-DIRNAME = path.dirname(__file__)
-ICON_BLANK = path.join(DIRNAME, "resources/blank-1-24.png")
-ICON_APP = path.join(DIRNAME, "resources/iconmonstr-location-27-32.png")
-ICON_CONN = path.join(DIRNAME, "resources/iconmonstr-media-control-48-24.png")
-ICON_SERIAL = path.join(DIRNAME, "resources/usbport-1-24.png")
-ICON_SOCKET = path.join(DIRNAME, "resources/ethernet-1-24.png")
-ICON_DISCONN = path.join(DIRNAME, "resources/iconmonstr-media-control-50-24.png")
-ICON_POS = path.join(DIRNAME, "resources/iconmonstr-location-1-24.png")
-ICON_SEND = path.join(DIRNAME, "resources/iconmonstr-arrow-12-24.png")
-ICON_EXIT = path.join(DIRNAME, "resources/iconmonstr-door-6-24.png")
-ICON_PENDING = path.join(DIRNAME, "resources/iconmonstr-time-6-24.png")
-ICON_CONFIRMED = path.join(DIRNAME, "resources/iconmonstr-check-mark-8-24.png")
-ICON_WARNING = path.join(DIRNAME, "resources/iconmonstr-warning-1-24.png")
-ICON_UNKNOWN = path.join(DIRNAME, "resources/clear-1-24.png")
-ICON_UBXCONFIG = path.join(DIRNAME, "resources/iconmonstr-gear-2-24.png")
-ICON_NTRIPCONFIG = path.join(DIRNAME, "resources/iconmonstr-antenna-4-24.png")
-ICON_SPARTNCONFIG = path.join(DIRNAME, "resources/iconmonstr-antenna-3-24.png")
-ICON_LOGREAD = path.join(DIRNAME, "resources/binary-1-24.png")
-ICON_REFRESH = path.join(DIRNAME, "resources/iconmonstr-refresh-6-16.png")
-ICON_CONTRACT = path.join(DIRNAME, "resources/iconmonstr-triangle-1-16.png")
-ICON_EXPAND = path.join(DIRNAME, "resources/iconmonstr-arrow-80-16.png")
-ICON_TRANSMIT = path.join(DIRNAME, "resources/iconmonstr-transmit-10-24.png")
-ICON_NOTRANSMIT = path.join(DIRNAME, "resources/iconmonstr-notransmit-10-24.png")
-ICON_NOCLIENT = path.join(DIRNAME, "resources/iconmonstr-noclient-10-24.png")
-ICON_LOAD = path.join(DIRNAME, "resources/iconmonstr-folder-18-24.png")
-ICON_SAVE = path.join(DIRNAME, "resources/iconmonstr-save-14-24.png")
-ICON_PLAY = path.join(DIRNAME, "resources/iconmonstr-media-control-48-24.png")
-ICON_RECORD = path.join(DIRNAME, "resources/iconmonstr-record-24.png")
-ICON_STOP = path.join(DIRNAME, "resources/iconmonstr-stop-1-24.png")
-ICON_UNDO = path.join(DIRNAME, "resources/iconmonstr-undo-24.png")
-ICON_DELETE = path.join(DIRNAME, "resources/iconmonstr-trash-can-filled-24.png")
-ICON_REDRAW = path.join(DIRNAME, "resources/iconmonstr-refresh-lined-24.png")
-IMG_WORLD = path.join(DIRNAME, "resources/world.png")
 
-# dialog task names
+BADCOL = "red"
+BGCOL = "gray24"  # default widget background color
+BPSRATES = (
+    9600,
+    19200,
+    38400,
+    57600,
+    115200,
+    230400,
+    460800,
+    921600,
+    1000000,
+    4800,
+)
+CFG = "cfg"
+CHECK_FOR_UPDATES = False
+CLASS = "cls"
+COLORTAGS = "colortags"
+CONFIGNAME = "pygpsclient"
+CONFIGFILE = path.join(HOME, f"{CONFIGNAME}.json")
+CONNECTED = 1
+CONNECTED_FILE = 4
+CONNECTED_NTRIP = 8
+CONNECTED_SOCKET = 2
+CONNECTED_SPARTNIP = 16
+CONNECTED_SPARTNLB = CONNECTED
+CONNMODES = {
+    0: "not connected",
+    1: "serial",
+    2: "file",
+    3: "no serial devices",
+    4: "socket",
+}
+CRLF = b"\x0d\x0a"
+DDD = "DD.D"
+DEFAULT_BUFSIZE = 4096
+DEFAULT_PORT = 50010
+DEFAULT_SERVER = "localhost"
+DIRNAME = path.dirname(__file__)
+DISCONNECTED = 0
+DLG = "dlg"
 DLGTABOUT = "About"
-DLGTUBX = "UBX Configuration"
+DLGTGPX = "GPX Track Viewer"
 DLGTNTRIP = "NTRIP Configuration"
 DLGTSPARTN = "SPARTN Configuration"
-DLGTGPX = "GPX Track Viewer"
-THD = "thd"
-DLG = "dlg"
-FRM = "frm"
-
-# URLS
-PYPI_URL = "https://pypi.org/pypi/PyGPSClient"
+DLGTUBX = "UBX Configuration"
+DMM = "DM.M"
+DMS = "D.M.S"
+ECEF = "ECEF"
+ENABLE_CFG_OTHER = True  # enable CFG=* Other Configuration command panel
+EPILOG = "© 2022 SEMU Consulting BSD 3-Clause license - https://github.com/semuconsulting/pygpsclient/"
+ERRCOL = "pink"  # default invalid data entry field background color
+ERROR = "ERR!"
+FGCOL = "white"  # default widget foreground color
+FILEREAD_INTERVAL = 0.02  # delay between successive datalog file reads (seconds)
+FONT_FIXED = "TkFixedFont"
+FONT_MENU = "TkMenuFont"
+FONT_TEXT = "TkTextFont"
+FORMAT_BINARY = "Binary"
+FORMAT_BOTH = "Parsed + Hex Tabular"
+FORMAT_HEXSTR = "Hex String"
+FORMAT_HEXTAB = "Hex Tabular"
+FORMAT_PARSED = "Parsed"
+FORMATS = (
+    FORMAT_PARSED,
+    FORMAT_BINARY,
+    FORMAT_HEXSTR,
+    FORMAT_HEXTAB,
+    FORMAT_BOTH,
+)
+FRAME = "frm"
+GGA_INTERVALS = ("None", "2", "5", "10", "60", "120")
 GITHUB_URL = "https://github.com/semuconsulting/PyGPSClient"
-MAPQURL = "https://www.mapquestapi.com/staticmap/v5/map?key={}"
-MAPQTIMEOUT = 5
-MAPURL = (
-    MAPQURL
-    + "&locations={},{}&zoom={}&size={},{}&defaultMarker=marker-sm-616161-ff4444"
-    + "&shape=radius:{}|weight:1|fill:ccffff50|border:88888850|{},{}&scalebar={}"
-)
-GPXMAPURL = (
-    MAPQURL
-    + "&locations={},{}||{},{}&zoom={}&size={},{}&defaultMarker=marker-num"
-    + "&shape=weight:2|border:{}|{}&scalebar={}|bottom"
-)
-
-# how frequently the mapquest api is called to update the web map (seconds)
-MAP_UPDATE_INTERVAL = 60
-MIN_UPDATE_INTERVAL = 5
-XML_HDR = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
+GLONASS_NMEA = True  # use GLONASS NMEA SVID (65-96) rather than slot (1-24)
+GNSS_EOF_EVENT = "<<gnss_eof>>"
+GNSS_ERR_EVENT = "<<gnss_error>>"
+GNSS_EVENT = "<<gnss_read>>"
+GNSS_LIST = {
+    0: ("GPS", "royalblue"),
+    1: ("SBA", "orange"),
+    2: ("GAL", "green4"),
+    3: ("BEI", "mediumpurple2"),
+    4: ("IME", "violet"),
+    5: ("QZS", "yellow"),
+    6: ("GLO", "indianred"),
+    7: ("NAV", "grey60"),
+}
+GPSEPOCH0 = datetime(1980, 1, 6)  # for Wno and Tow calculations
 GPX_NS = " ".join(
     (
         'xmlns="http://www.topografix.com/GPX/1/1"',
@@ -95,24 +121,112 @@ GPX_NS = " ".join(
         'http://www.topografix.com/GPX/1/1/gpx.xsd"',
     )
 )
-EPILOG = "© 2022 SEMU Consulting BSD 3-Clause license - https://github.com/semuconsulting/pygpsclient/"
-GPSEPOCH0 = datetime(1980, 1, 6)  # for Wno and Tow calculations
-GPXLIMIT = 500  # max number of track points supported by MapQuest API
-# Check for newer PyPi version on startup (requires internet connection).
-# NB: the PyPi API is not very quick so enabling this will increase
-# application startup time.
-CHECK_FOR_UPDATES = False
-GUI_UPDATE_INTERVAL = 0.5  # minimum GUI widget update interval (seconds)
 GPX_TRACK_INTERVAL = 1  # minimum GPS track update interval (seconds)
-FILEREAD_INTERVAL = 0.02  # delay between successive datalog file reads (seconds)
-SAT_EXPIRY = 10  # how long passed satellites are kept in the sky and graph views
+GUI_UPDATE_INTERVAL = 0.5  # minimum GUI widget update interval (seconds)
+ICON_APP = path.join(DIRNAME, "resources/iconmonstr-location-27-32.png")
+ICON_BLANK = path.join(DIRNAME, "resources/blank-1-24.png")
+ICON_CONFIRMED = path.join(DIRNAME, "resources/iconmonstr-check-mark-8-24.png")
+ICON_CONN = path.join(DIRNAME, "resources/iconmonstr-media-control-48-24.png")
+ICON_CONTRACT = path.join(DIRNAME, "resources/iconmonstr-triangle-1-16.png")
+ICON_DELETE = path.join(DIRNAME, "resources/iconmonstr-trash-can-filled-24.png")
+ICON_DISCONN = path.join(DIRNAME, "resources/iconmonstr-media-control-50-24.png")
+ICON_EXIT = path.join(DIRNAME, "resources/iconmonstr-door-6-24.png")
+ICON_EXPAND = path.join(DIRNAME, "resources/iconmonstr-arrow-80-16.png")
+ICON_LOAD = path.join(DIRNAME, "resources/iconmonstr-folder-18-24.png")
+ICON_LOGREAD = path.join(DIRNAME, "resources/binary-1-24.png")
+ICON_NOCLIENT = path.join(DIRNAME, "resources/iconmonstr-noclient-10-24.png")
+ICON_NOTRANSMIT = path.join(DIRNAME, "resources/iconmonstr-notransmit-10-24.png")
+ICON_NTRIPCONFIG = path.join(DIRNAME, "resources/iconmonstr-antenna-4-24.png")
+ICON_PENDING = path.join(DIRNAME, "resources/iconmonstr-time-6-24.png")
+ICON_PLAY = path.join(DIRNAME, "resources/iconmonstr-media-control-48-24.png")
+ICON_POS = path.join(DIRNAME, "resources/iconmonstr-location-1-24.png")
+ICON_RECORD = path.join(DIRNAME, "resources/iconmonstr-record-24.png")
+ICON_REDRAW = path.join(DIRNAME, "resources/iconmonstr-refresh-lined-24.png")
+ICON_REFRESH = path.join(DIRNAME, "resources/iconmonstr-refresh-6-16.png")
+ICON_SAVE = path.join(DIRNAME, "resources/iconmonstr-save-14-24.png")
+ICON_SEND = path.join(DIRNAME, "resources/iconmonstr-arrow-12-24.png")
+ICON_SERIAL = path.join(DIRNAME, "resources/usbport-1-24.png")
+ICON_SOCKET = path.join(DIRNAME, "resources/ethernet-1-24.png")
+ICON_SPARTNCONFIG = path.join(DIRNAME, "resources/iconmonstr-antenna-3-24.png")
+ICON_STOP = path.join(DIRNAME, "resources/iconmonstr-stop-1-24.png")
+ICON_TRANSMIT = path.join(DIRNAME, "resources/iconmonstr-transmit-10-24.png")
+ICON_UBXCONFIG = path.join(DIRNAME, "resources/iconmonstr-gear-2-24.png")
+ICON_UNDO = path.join(DIRNAME, "resources/iconmonstr-undo-24.png")
+ICON_UNKNOWN = path.join(DIRNAME, "resources/clear-1-24.png")
+ICON_WARNING = path.join(DIRNAME, "resources/iconmonstr-warning-1-24.png")
+IMG_WORLD = path.join(DIRNAME, "resources/world.png")
+KM2M = 1000
+KM2MIL = 0.621371
+KM2NMIL = 0.5399568
+KNOWNGPS = (
+    "CP210",
+    "FT230",
+    "FT232",
+    "garmin",
+    "Garmin",
+    "gnss",
+    "GNSS",
+    "gps",
+    "GPS",
+    "magellan",
+    "Magellan",
+    "sirf",
+    "Sirf",
+    "SiRF",
+    "trimble",
+    "Trimble",
+    "u-blox",
+    "U-Blox",
+    "ublox",
+    "USB to UART",
+    "USB UART",
+    "USB_to_UART",
+    "USER-DEFINED",
+)
+KPH2KNT = 0.5399568
+KPH2MPH = 0.621371
+KPH2MPS = 0.2777776918389111005
+M2FT = 3.28084
 MAX_SNR = 60  # upper limit of graphview snr axis
 MAXLOGLINES = 10000  # maximum number of 'lines' per datalog file
-# default error handling behaviour for UBXReader.read() calls
-# 0 (ERR_IGNORE) = ignore errors, 1 (ERR_LOG) - log errors, 2 (ERR_RAISE) = raise errors
-QUITONERRORDEFAULT = 1
+MQAPIKEY = "mqapikey"
+MSGMODES = {
+    "GET": GET,
+    "SET": SET,
+    "POLL": POLL,
+}
+NOPORTS = 3
+NTRIP_EVENT = "<<ntrip_read>>"
+OKCOL = "green"
 POPUP_TRANSIENT = True  # whether pop-up config dialogs are always on top
 PORTIDS = ("0 I2C", "1 UART1", "2 UART2", "3 USB", "4 SPI")
+PYPI_URL = "https://pypi.org/pypi/PyGPSClient"
+QUITONERRORDEFAULT = 1
+RCVR_CONNECTION = "USB"  # default GNSS receiver connection port
+READONLY = "readonly"
+RXMMSG = "RXM-SPARTN-KEY"
+SAT_EXPIRY = 10  # how long passed satellites are kept in the sky and graph view
+SAVED_CONFIG = "saved_config"
+SOCK_NTRIP = "NTRIP CASTER"
+SOCK_SERVER = "SOCKET SERVER"
+SOCKCLIENT_HOST = "localhost"
+SOCKCLIENT_PORT = 50010
+SOCKMODES = (SOCK_SERVER, SOCK_NTRIP)
+SOCKSERVER_HOST = "0.0.0.0"  # i.e. bind to all host IP addresses
+SOCKSERVER_MAX_CLIENTS = 5
+SOCKSERVER_NTRIP_PORT = 2101
+SOCKSERVER_PORT = 50012
+SPARTN_EOF_EVENT = "<<spartn_eof>>"
+SPARTN_ERR_EVENT = "<<spartn_error>>"
+SPARTN_EVENT = "<<spartn_read>>"
+SPARTN_KEYLEN = 16
+SPARTN_OUTPORT = 8883
+SPARTN_PPREGIONS = ("eu", "us", "kr", "au")
+SPARTN_PPSERVER = "pp.services.u-blox.com"
+SPARTN_PPSERVER = "pp.services.u-blox.com"
+SPARTN_SOURCE_IP = 0
+SPARTN_SOURCE_LB = 1
+THD = "thd"
 TIMEOUTS = (
     "0.1",
     "0.2",
@@ -124,113 +238,19 @@ TIMEOUTS = (
     "None",
     "0",
 )
-ANTSTATUS = ("INIT", "DONTKNOW", "OK", "SHORT", "OPEN")
-ANTPOWER = ("OFF", "ON", "DONTKNOW")
-GGA_INTERVALS = ("None", "2", "5", "10", "60", "120")
-ENABLE_CFG_OTHER = True  # enable CFG=* Other Configuration command panel
-# names of user config and preset files:
-CONFIGNAME = "pygpsclient"
-CONFIGFILE = path.join(HOME, f"{CONFIGNAME}.json")
-MQAPIKEY = "mqapikey"
+TOPIC_IP = "/pp/ip/{}"
+TOPIC_MGA = "/pp/ubx/mga"
+TOPIC_RXM = "/pp/ubx/0236/ip"
 UBXPRESETS = "ubxpresets"
-COLORTAGS = "colortags"
-# list of recognised serial device descriptors:
-KNOWNGPS = (
-    "USER-DEFINED",
-    "GPS",
-    "gps",
-    "GNSS",
-    "gnss",
-    "Garmin",
-    "garmin",
-    "U-Blox",
-    "u-blox",
-    "ublox",
-    "SiRF",
-    "Sirf",
-    "sirf",
-    "Magellan",
-    "magellan",
-    "CP210",
-    "FT232",
-    "USB UART",
-    "USB to UART",
-    "USB_to_UART",
-)
-# list of available bps rates (first entry in list is the default):
-BPSRATES = (9600, 19200, 38400, 57600, 115200, 230400, 460800, 921600, 1000000, 4800)
-# terminator for NMEA protocol
-CRLF = b"\x0d\x0a"
-# socket client defaults
-DEFAULT_SERVER = "localhost"
-DEFAULT_PORT = 50010
-DEFAULT_BUFSIZE = 4096
-# socket server defaults
-SOCKSERVER_HOST = "0.0.0.0"  # i.e. bind to all host IP addresses
-SOCKSERVER_PORT = 50010
-SOCKSERVER_NTRIP_PORT = 2101
-SOCKSERVER_MAX_CLIENTS = 5
-# display formats
-FORMAT_PARSED = 1
-FORMAT_BIN = 2
-FORMAT_HEX = 4
-FORMAT_HEXTABLE = 8
-FORMATS = ("Parsed", "Binary", "Hex String", "Hex Tabular", "Parsed + Hex Tabular")
-# socket server modes
-SOCKMODES = ("SOCKET SERVER", "NTRIP CASTER")
-# connection mode flags:
-DISCONNECTED = 0
-CONNECTED = 1
-CONNECTED_SOCKET = 2
-CONNECTED_FILE = 4
-NOPORTS = 3
-CONNMODES = {
-    0: "not connected",
-    1: "serial",
-    2: "file",
-    3: "no serial devices",
-    4: "socket",
-}
-MSGMODES = {
-    "GET": GET,
-    "SET": SET,
-    "POLL": POLL,
-}
-# read event types
-GNSS_EVENT = "<<gnss_read>>"
-GNSS_EOF_EVENT = "<<gnss_eof>>"
-NTRIP_EVENT = "<<ntrip_read>>"
-SPARTN_EVENT = "<<spartn_read>>"
-SPARTN_EOF_EVENT = "<<spartn_eof>>"
-# default widget frame sizes:
+UI = "Imperial mph"
+UIK = "Imperial knots"
+UMK = "Metric kmph"
+UMM = "Metric m/s"
 WIDGETU1 = (250, 250)  # small widget size
 WIDGETU2 = (350, 250)  # medium widget size
 WIDGETU3 = (950, 350)  # Console size
 WIDGETU4 = (600, 600)  # GPX Track viewer size
-
-BGCOL = "gray24"  # default widget background color
-FGCOL = "white"  # default widget foreground color
-# ENTCOL = "azure"  # default valid data entry field background color
-ERRCOL = "pink"  # default invalid data entry field background color
-# INFCOL = "steelblue3"  # readonly info field text color
-OKCOL = "green"
-BADCOL = "red"
-READONLY = "readonly"
-ERROR = "ERR!"
-DDD = "DD.D"
-DMM = "DM.M"
-DMS = "D.M.S"
-UMM = "Metric m/s"
-UMK = "Metric kmph"
-UI = "Imperial mph"
-UIK = "Imperial knots"
-KPH2MPH = 0.621371
-KPH2KNT = 0.5399568
-KPH2MPS = 0.2777776918389111005
-M2FT = 3.28084
-KM2M = 1000
-KM2MIL = 0.621371
-KM2NMIL = 0.5399568
+XML_HDR = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
 
 # UBX config widget signifiers - used to
 # identify which widget should receive the
@@ -247,36 +267,15 @@ SPARTN_GNSS = 8
 SPARTN_LBAND = 9
 SPARTN_MQTT = 10
 SPECTRUMVIEW = 11
-
-# SPARTN globals
-SPARTN_SOURCE_LB = 1
-SPARTN_SOURCE_IP = 0
-SPARTN_PPSERVER = "pp.services.u-blox.com"
-OUTPORT_SPARTN = 8883
-SPARTN_PPREGIONS = ("eu", "us", "kr", "au")
-SPARTN_KEYLEN = 16
-RXMMSG = "RXM-SPARTN-KEY"
-CONNECTED_NTRIP = 8
-CONNECTED_SPARTNIP = 16
-CONNECTED_SPARTNLB = CONNECTED
-TOPIC_RXM = "/pp/ubx/0236/ip"
-TOPIC_MGA = "/pp/ubx/mga"
-TOPIC_IP = "/pp/ip/{}"
-
-GLONASS_NMEA = True  # use GLONASS NMEA SVID (65-96) rather than slot (1-24)
-# GNSS color codings:
-GNSS_LIST = {
-    0: ("GPS", "royalblue"),
-    1: ("SBA", "orange"),
-    2: ("GAL", "green4"),
-    3: ("BEI", "mediumpurple2"),
-    4: ("IME", "violet"),
-    5: ("QZS", "yellow"),
-    6: ("GLO", "indianred"),
-}
+SYSMONVIEW = 12
+ROVERVIEW = 13
 
 # map of fix values to descriptions
-# (for NMEA 4 and above)
+# the keys in this map are a concatenation of NMEA/UBX
+# message identifier and attribute value e.g.:
+# GGA1: GGA + quality = 1
+# NAV-STATUS3: NAV-STATUS + gpsFix = 3
+# (valid for NMEA >=4)
 FIXLOOKUP = {
     "GGA1": "3D",  # quality
     "GGA2": "3D",
@@ -299,19 +298,19 @@ FIXLOOKUP = {
     "VTGA": "3D",  # posMode
     "VTGD": "RTK",
     "VTGE": "DR",
-    "NAV-PVT1": "DR",  # fixType or carrSoln
+    "NAV-PVT1": "DR",  # fixType
     "NAV-PVT2": "2D",
     "NAV-PVT3": "3D",
     "NAV-PVT4": "GNSS+DR",
     "NAV-PVT5": "TIME ONLY",
-    "NAV-PVT6": "RTK FLOAT",
+    "NAV-PVT6": "RTK FLOAT",  # carrSoln
     "NAV-PVT7": "RTK FIXED",
-    "NAV-STATUS1": "DR",  # gpsFix or carrSoln
+    "NAV-STATUS1": "DR",  # gpsFix
     "NAV-STATUS2": "3D",
     "NAV-STATUS3": "3D",
     "NAV-STATUS4": "GNSS+DR",
     "NAV-STATUS5": "TIME ONLY",
-    "NAV-STATUS6": "RTK FLOAT",
+    "NAV-STATUS6": "RTK FLOAT",  # carrSoln
     "NAV-STATUS7": "RTK FIXED",
     "NAV-SOL1": "DR",  # gpsFix
     "NAV-SOL2": "3D",
@@ -324,25 +323,18 @@ FIXLOOKUP = {
     "HNR-PVT4": "GNSS+DR",
     "HNR-PVT5": "TIME ONLY",
     "HNR-PVT6": "RTK",
-    "NAV2-PVT1": "DR",  # fixType or carrSoln
+    "NAV2-PVT1": "DR",  # fixType
     "NAV2-PVT2": "2D",
     "NAV2-PVT3": "3D",
     "NAV2-PVT4": "GNSS+DR",
     "NAV2-PVT5": "TIME ONLY",
-    "NAV2-PVT6": "RTK FLOAT",
+    "NAV2-PVT6": "RTK FLOAT",  # carrSoln
     "NAV2-PVT7": "RTK FIXED",
-    "NAV2-STATUS1": "DR",  # gpsFix or carrSoln
+    "NAV2-STATUS1": "DR",  # gpsFix
     "NAV2-STATUS2": "3D",
     "NAV2-STATUS3": "3D",
     "NAV2-STATUS4": "GNSS+DR",
     "NAV2-STATUS5": "TIME ONLY",
-    "NAV2-STATUS6": "RTK FLOAT",
+    "NAV2-STATUS6": "RTK FLOAT",  # carrSoln
     "NAV2-STATUS7": "RTK FIXED",
 }
-
-FONT_MENU = "TkMenuFont"
-FONT_TEXT = "TkTextFont"
-FONT_FIXED = "TkFixedFont"
-TAG_COLORS = False  # default colortag setting
-
-# TAGS are now in 'colortags' file in user's home directory
