@@ -1,7 +1,9 @@
 """
-PyGPSClient Helpers
+helpers.py
 
-Collection of helper methods
+PyGPSClient Helpers.
+
+Collection of helper methods.
 
 Created on 17 Apr 2021
 
@@ -12,21 +14,24 @@ Created on 17 Apr 2021
 """
 
 import os
-import re
 from datetime import datetime, timedelta
-from math import cos, pi, sin
+from math import atan, cos, pi, sin, sqrt, trunc
+from platform import system
 from time import strftime
-from tkinter import Button, Entry, Label, Toplevel, W
+from tkinter import Button, Entry, Label, Toplevel, W, font
 
 from pynmeagps import haversine
-from pyubx2 import UBXMessage, attsiz, atttyp
+from pyubx2 import SET, UBX_MSGIDS, UBXMessage, attsiz, atttyp
 from requests import get
 
-from pygpsclient.globals import FIXLOOKUP, GPSEPOCH0, MAX_SNR
+from pygpsclient.globals import FIXLOOKUP, GPSEPOCH0, MAX_SNR, RCVR_CONNECTION
+from pygpsclient.strings import NA
 
 # validation type flags
 MAXPORT = 65535
 MAXALT = 10000.0  # meters arbitrary
+MAXFLOAT = 2e20
+MINFLOAT = -MAXFLOAT
 VALBLANK = 1
 VALNONBLANK = 2
 VALINT = 4
@@ -36,6 +41,12 @@ VALHEX = 32
 VALDMY = 64
 VALLEN = 128
 
+# NTRIP enumerations
+NMEA = {"0": "No GGA", "1": "GGA"}
+AUTHS = {"N": "None", "B": "Basic", "D": "Digest"}
+CARRIERS = {"0": "No", "1": "L1", "2": "L1,L2"}
+SOLUTIONS = {"0": "Single", "1": "Network"}
+
 
 class ConfirmBox(Toplevel):
     """
@@ -396,11 +407,47 @@ def svid2gnssid(svid) -> int:
         gnssId = 5  # QZSS
     elif (65 <= svid <= 96) or svid == 255:
         gnssId = 6  # GLONASS
+    elif 247 <= svid <= 253:
+        gnssId = 7  # NAVIC
     else:
         gnssId = 0  # GPS
     return gnssId
 
 
+def get_mp_info(srt: list) -> dict:
+    """
+    Get mountpoint information from sourcetable entry.
+
+    :param list srt: sourcetable entry as list
+    :return: dictionary of mountpoint info
+    :rtype: dict or None if not available
+    """
+
+    try:
+        info = {
+            "name": srt[0],
+            "identifier": srt[1],
+            "format": srt[2],
+            "messages": srt[3],
+            "carrier": CARRIERS[srt[4]],
+            "navs": srt[5],
+            "network": srt[6],
+            "country": srt[7],
+            "lat": srt[8],
+            "lon": srt[9],
+            "gga": NMEA[srt[10]],
+            "solution": SOLUTIONS[srt[11]],
+            "generator": srt[12],
+            "encrypt": srt[13],
+            "auth": AUTHS[srt[14]],
+            "fee": srt[15],
+            "bitrate": srt[16],
+        }
+        return info
+    except IndexError:
+        return None
+
+
 def get_mp_distance(lat: float, lon: float, mp: list) -> float:
     """
     Get distance to mountpoint from current location (if known).
@@ -442,7 +489,7 @@ def check_latest(name: str) -> str:
             "version"
         ]
     except Exception:  # pylint: disable=broad-except
-        return "N/A"
+        return NA
 
 
 def fix2desc(msgid: str, fix: object) -> str:
@@ -495,20 +542,21 @@ def validURL(url: str) -> bool:
     """
     # pylint: disable=line-too-long
 
-    regex = re.compile(
-        # r"^(?:http|https)?://"  # http:// or https://
-        r"^(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|"  # domain...
-        r"localhost|"  # localhost...
-        r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"  # ...or ip
-        r"(?::\d+)?"  # optional port
-        r"(?:/?|[/?]\S+)$",
-        re.IGNORECASE,
-    )
+    # regex = re.compile(
+    #     # r"^(?:http|https)?://"  # http:// or https://
+    #     r"^(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|"  # domain...
+    #     r"localhost|"  # localhost...
+    #     r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})"  # ...or ip
+    #     r"(?::\d+)?"  # optional port
+    #     r"(?:/?|[/?]\S+)$",
+    #     re.IGNORECASE,
+    # )
 
-    return re.match(regex, url) is not None
+    # return re.match(regex, url) is not None
+    return url not in (None, "")
 
 
-def valid_entry(entry: Entry, valmode: int, low=None, high=None) -> bool:
+def valid_entry(entry: Entry, valmode: int, low=MINFLOAT, high=MAXFLOAT) -> bool:
     """
     Validates tkinter entry field and highlights it if in error.
 
@@ -548,7 +596,7 @@ def valid_entry(entry: Entry, valmode: int, low=None, high=None) -> bool:
         entry.configure(highlightthickness=0)
     else:
         entry.configure(
-            highlightthickness=1, highlightbackground="red", highlightcolor="red"
+            highlightthickness=2, highlightbackground="red", highlightcolor="red"
         )
     return valid
 
@@ -677,108 +725,204 @@ def parse_rxmspartnkey(msg: UBXMessage) -> list:
     return keys
 
 
-# ------------------------------------------------------------------
-# FOLLOWING MAPQUEST POLYGON COMPRESSION AND DECOMPRESSION ROUTINES
-# ADAPTED FROM THE ORIGINAL JAVASCRIPT EXAMPLES:
-# https://developer.mapquest.com/documentation/common/encode-decode/
-# ------------------------------------------------------------------
+def bytes2unit(valb: int) -> tuple:
+    """Format bytes as KB, MB, GB etc
+    such that value < 100.
 
-
-def mapq_encode(num: int) -> str:
-    """
-    Encode number representing character.
-
-    :param int num: number to encode
-    :return: encoded number as string
-    :rtype: str
+    :param int valb: bytes
+    :return: tuple of (value, units)
     """
 
-    num = num << 1
-    if num < 0:
-        num = ~(num)
+    if not isinstance(valb, (int, float)):
+        return 0, NA
 
-    encoded = ""
-    while num >= 0x20:
-        encoded += chr((0x20 | (num & 0x1F)) + 63)
-        num >>= 5
-
-    encoded += chr(num + 63)
-    return encoded
+    BYTESUNITS = ["", "KB", "MB", "GB", "TB"]
+    i = 0
+    val = valb
+    valu = BYTESUNITS[i]
+    while val > 500:
+        val = valb / (2 ** (i * 10))
+        valu = BYTESUNITS[i]
+        i += 1
+        if i > 4:
+            break
+    return val, valu
 
 
-def mapq_decompress(encoded: str, precision: int = 6) -> list:
+def secs2unit(secs: int) -> tuple:
     """
-    Decompress polygon for MapQuest API.
+    Format seconds as secs, mins, hours or days
+    such that value is > 100.
 
-    :param str encoded: polygon encoded as string
-    :param int precision: no decimal places precision (6)
-    :return: polygon as list of points
-    :rtype: list
+    :param int secs: seconds
+    :return: tuple of (value, units)
+    :rtype: tuple
     """
 
-    precision = 10**-precision
-    leng = len(encoded)
-    index = 0
-    lat = 0
-    lng = 0
-    array = []
-    while index < leng:
-        shift = 0
-        result = 0
-        b = 0xFF
-        while b >= 0x20:
-            b = ord(encoded[index]) - 63
-            index += 1
-            result |= (b & 0x1F) << shift
-            shift += 5
+    if not isinstance(secs, (int, float)):
+        return 0, NA
 
-        dlat = ~(result >> 1) if (result & 1) else (result >> 1)
-        lat += dlat
-        shift = 0
-        result = 0
-        b = 0xFF
-        while b >= 0x20:
-            b = ord(encoded[index]) - 63
-            index += 1
-            result |= (b & 0x1F) << shift
-            shift += 5
+    SECSUNITS = ["secs", "mins", "hrs", "days"]
+    SECSDIV = [60, 3600, 86400]
 
-        dlng = ~(result >> 1) if (result & 1) else (result >> 1)
-        lng += dlng
-        array.append(lat * precision)
-        array.append(lng * precision)
-
-    return array
+    i = 0
+    val = secs
+    while val > 100:
+        val = secs / SECSDIV[i]
+        i += 1
+        if i > 2:
+            break
+    return val, SECSUNITS[i]
 
 
-def mapq_compress(points: list, precision: int = 6) -> str:
+def sizefont(height: int, lines: int, minfont: int) -> tuple:
     """
-    Compress polygon for MapQuest API.
+    Set font size according to number of text lines on widget
+    of given height.
 
-    :param str points: polygon as list of points
-    :param int precision: no decimal places precision (6)
-    :return: polygon encoded as string
-    :rtype: string
+    :param int maxlines: max no of lines of text
+    :param int minfont: min font size
+    :returns: tuple of (font, fontheight)
+    :rtype: tuple
     """
 
-    oldLat = 0
-    oldLng = 0
-    leng = len(points)
-    index = 0
-    encoded = ""
-    precision = 10**precision
-    while index < leng:
-        #  Round to N decimal places
-        lat = round(points[index] * precision)
-        index += 1
-        lng = round(points[index] * precision)
-        index += 1
+    fh = 0
+    fs = minfont
+    while fh * lines < height:
+        fnt = font.Font(size=fs)
+        fh = fnt.metrics("linespace")
+        fs += 1
+    return fnt, fh
 
-        #  Encode the differences between the points
-        encoded += mapq_encode(lat - oldLat)
-        encoded += mapq_encode(lng - oldLng)
 
-        oldLat = lat
-        oldLng = lng
+def setubxrate(app: object, mid: str, rate: int = 1) -> UBXMessage:
+    """
+    Set rate on specified UBX message on default port(s).
 
-    return encoded
+    The port(s) this applies to are defined in the 'defaultport_s'
+    configuration setting as a comma-separated string.
+
+    Rate is relative to navigation solution e.g.
+    a rate of '4' means 'every 4th navigation solution'
+    (higher = less frequent).
+
+    :param object application: reference to calling application
+    :param str mid: message identity e.g. "MON-SPAN"
+    :param int rate: message rate (0 = off)
+    :return: UBX command generated
+    :rtype: UBXMessage
+    :raises: ValueError if unknown message identity
+    """
+
+    rates = {}
+    prts = app.frm_settings.config.get(
+        "defaultport_s", app.frm_settings.config.get("defaultport", RCVR_CONNECTION)
+    ).split(",")
+    for prt in prts:
+        rates[prt] = rate
+
+    msgclass = msgid = None
+    for msgtype, msgnam in UBX_MSGIDS.items():
+        if msgnam == mid:
+            msgclass = msgtype[0]
+            msgid = msgtype[1]
+            break
+    if msgclass is None or msgid is None:
+        raise ValueError(f"Message ID {mid} unknown")
+
+    msg = UBXMessage(
+        "CFG",
+        "CFG-MSG",
+        SET,
+        msgClass=msgclass,
+        msgID=msgid,
+        rateDDC=rates.get("I2C", 0),
+        rateUART1=rates.get("UART1", 0),
+        rateUART2=rates.get("UART2", 0),
+        rateUSB=rates.get("USB", 0),
+        rateSPI=rates.get("SPI", 0),
+    )
+    app.gnss_outqueue.put(msg.serialize())
+    return msg
+
+
+def val2sphp(val: float, scale: float) -> tuple:
+    """
+    Convert a float value into separate
+    standard and high precisions components,
+    multiplied by a scaling factor to render them
+    as integers.
+
+    As required by e.g. UBX CFG-TMODE command.
+
+    :param float val: value as float
+    :param float scale: scaling factor e.g. 1e-7
+    :return: tuple of (standard precision, high precision)
+    :rtype: tuple
+    """
+
+    val = val / scale
+    val_sp = trunc(val)
+    val_hp = round((val - val_sp) * 100)
+    return val_sp, val_hp
+
+
+def config_nmea(state: int, port_type: str = "USB") -> UBXMessage:
+    """
+    Enable or disable NMEA messages at port level and use minimum UBX
+    instead (NAV-PRT, NAV_SAT, NAV_DOP).
+
+    :param int state: 1 = disable NMEA, 0 = enable NMEA
+    :param str port_type: port that rcvr is connected on
+    """
+
+    nmea_state = 0 if state else 1
+    layers = 1
+    transaction = 0
+    cfg_data = []
+    cfg_data.append((f"CFG_{port_type}OUTPROT_NMEA", nmea_state))
+    cfg_data.append((f"CFG_{port_type}OUTPROT_UBX", 1))
+    cfg_data.append((f"CFG_MSGOUT_UBX_NAV_PVT_{port_type}", state))
+    cfg_data.append((f"CFG_MSGOUT_UBX_NAV_DOP_{port_type}", state))
+    cfg_data.append((f"CFG_MSGOUT_UBX_NAV_SAT_{port_type}", state * 4))
+
+    return UBXMessage.config_set(layers, transaction, cfg_data)
+
+
+def adjust_dimensions(dim: int) -> int:
+    """
+    Adjust display dimensions for different operating systems.
+
+    :param int dim: dimension
+    :return: adjusted dimension
+    :rtype: int
+    """
+
+    if system() == "Windows":
+        return int(dim * 0.95)
+    if system() == "Darwin":
+        return int(dim * 0.9)
+    return int(dim)
+
+
+def ned2vector(n: float, e: float, d: float) -> tuple:
+    """
+    Convert N,E,D relative position to 2D heading and distance.
+
+    :param float n: north coordinate
+    :param float e: east coordinate
+    :param float d: down coordinate
+    :return: tuple of distance, heading
+    :rtype: tuple
+    """
+
+    dis = sqrt(n**2 + e**2 + d**2)
+    if n == 0 or e == 0:
+        hdg = 0
+    else:
+        hdg = atan(e / n) * 180 / pi
+        if hdg > 0:
+            hdg += 180
+        else:
+            hdg += 360
+    return dis, hdg
