@@ -1,4 +1,6 @@
 """
+scatter_frame.py
+
 Scatterplot frame class for PyGPS Application.
 
 This generates a scatterplot of positions, centered on the average
@@ -14,7 +16,7 @@ statistics and geographiclib libraries.
 :license: BSD 3-Clause
 """
 
-from math import cos, radians, sin, sqrt
+from math import cos, radians, sin
 from tkinter import BOTH, HORIZONTAL, NO, YES, Frame, IntVar, Scale, font
 
 from pynmeagps import bearing, haversine
@@ -22,6 +24,8 @@ from pynmeagps import bearing, haversine
 from pygpsclient.globals import BGCOL, FGCOL, WIDGETU2, Point
 from pygpsclient.skyview_frame import Canvas
 
+SQRT2 = 0.7071067811865476
+
 
 class ScatterViewFrame(Frame):
     """
@@ -68,7 +72,8 @@ class ScatterViewFrame(Frame):
             to=len(self.scale_factors) - 1,
             orient=HORIZONTAL,
             relief="sunken",
-            bg=BGCOL,
+            bg=FGCOL,
+            troughcolor=BGCOL,
             variable=self.scale,
             showvalue=False,
             command=self._rescale,
@@ -99,7 +104,7 @@ class ScatterViewFrame(Frame):
         """
 
         self.width, self.height = self.get_size()
-        self.init_graph()
+        self.init_frame()
         self.redraw()
 
     def _on_clear(self, event):  # pylint: disable=unused-argument
@@ -110,7 +115,7 @@ class ScatterViewFrame(Frame):
         """
 
         self.points = []
-        self.init_graph()
+        self.init_frame()
 
     def get_size(self) -> tuple:
         """
@@ -146,7 +151,7 @@ class ScatterViewFrame(Frame):
             5, 10 + height, text=lon, fill=self.fg_col, font=lbl_font, anchor="w"
         )
 
-    def init_graph(self):
+    def init_frame(self):
         """
         Initialize scatter plot.
         """
@@ -168,8 +173,8 @@ class ScatterViewFrame(Frame):
             distance = m_per_circle * (idx + 1)
             if len(str(distance)) > 4:
                 distance = round(distance, 3)
-            txt_x = width / 2 + sqrt(2) / 2 * maxr * rad
-            txt_y = height / 2 + sqrt(2) / 2 * maxr * rad
+            txt_x = width / 2 + SQRT2 * maxr * rad
+            txt_y = height / 2 + SQRT2 * maxr * rad
             self.canvas.create_text(
                 txt_x, txt_y, text=f"{distance}m", fill=self.fg_col, font=lbl_font
             )
@@ -243,7 +248,7 @@ class ScatterViewFrame(Frame):
         middle = self._ave_pos()
         if self.mean is not None and self.mean != middle:
             self.mean = middle
-            self.init_graph()
+            self.init_frame()
             self.redraw()
         else:
             self.mean = middle
