@@ -1,5 +1,12 @@
 """
-SPARTN L-Band Correction receiver configuration dialog
+spartn_lband_frame.py
+
+SPARTN L-Band Correction receiver configuration dialog.
+
+This handles the configuration of the L-Band receiver
+(e.g. u-blox D9S) for the selected region.
+
+Supply initial settings via `config` keyword argument.
 
 Created on 26 Jan 2023
 
@@ -27,10 +34,12 @@ from tkinter import (
 )
 
 from PIL import Image, ImageTk
-from pyubx2 import POLL_LAYER_RAM, SET, SET_LAYER_RAM, TXN_NONE, UBXMessage
+from pyubx2 import GET, POLL_LAYER_RAM, SET, SET_LAYER_RAM, TXN_NONE, UBXMessage
+from serial import PARITY_NONE
 
 from pygpsclient.globals import (
     BPSRATES,
+    CONNECTED,
     CONNECTED_SPARTNIP,
     CONNECTED_SPARTNLB,
     DISCONNECTED,
@@ -47,7 +56,9 @@ from pygpsclient.globals import (
     MSGMODES,
     NOPORTS,
     READONLY,
+    SAVED_CONFIG,
     SPARTN_EOF_EVENT,
+    SPARTN_ERR_EVENT,
     SPARTN_EVENT,
     SPARTN_LBAND,
     TIMEOUTS,
@@ -72,7 +83,8 @@ PMP_DATARATES = {
     "B2400": 2400,
     "B4800": 4800,
 }
-SPARTN_DEFAULT = {
+D9S_FACTORY = {
+    "name": "D9S Factory Default",
     "freq": 1539812500,
     "schwin": 2200,
     "usesid": 1,
@@ -83,9 +95,35 @@ SPARTN_DEFAULT = {
     "descrminit": 23560,
     "unqword": 16238547128276412563,
 }
+D9S_PP_US = {
+    "name": "D9S PointPerfect US",
+    "freq": 1556290000,
+    "schwin": 2200,
+    "usesid": 0,
+    "sid": 21845,
+    "drat": PMP_DATARATES["B2400"],
+    "descrm": 1,
+    "prescrm": 0,
+    "descrminit": 26969,
+    "unqword": 16238547128276412563,
+}
+D9S_PP_EU = {
+    "name": "D9S PointPerfect EU",
+    "freq": 1545260000,
+    "schwin": 2200,
+    "usesid": 0,
+    "sid": 21845,
+    "drat": PMP_DATARATES["B2400"],
+    "descrm": 1,
+    "prescrm": 0,
+    "descrminit": 26969,
+    "unqword": 16238547128276412563,
+}
+# D9S default configuration
+D9S_CONFIG = D9S_PP_US  # D9S_FACTORY
 
 
-class SPARTNLBANDDialog(Frame):
+class SpartnLbandDialog(Frame):
     """,
     SPARTNConfigDialog class.
     """
@@ -105,6 +143,7 @@ class SPARTNLBANDDialog(Frame):
         self.__app = app  # Reference to main application class
         self.__master = self.__app.appmaster  # Reference to root class (Tk)
         self.__container = container  # container frame
+        self._saved_config = kwargs.pop(SAVED_CONFIG, {})
 
         Frame.__init__(self, self.__container.container, *args, **kwargs)
 
@@ -136,6 +175,7 @@ class SPARTNLBANDDialog(Frame):
 
         self._body()
         self._do_layout()
+        self._attach_events()
         self._reset()
 
     def _body(self):
@@ -145,14 +185,25 @@ class SPARTNLBANDDialog(Frame):
         # pylint: disable=unnecessary-lambda
         self._lbl_corrlband = Label(self, text=LBLSPARTNLB)
         # Correction receiver serial port configuration panel
-        userport = self.__app.app_config.get("spartnport", "")
+        saved_lband_config = {
+            "bpsrate_n": self.__app.saved_config.get("lbandclientbpsrate_n", 9600),
+            "databits_n": self.__app.saved_config.get("lbandclientdatabits_n", 8),
+            "stopbits_f": self.__app.saved_config.get("lbandclientstopbits_f", 1.0),
+            "parity_s": self.__app.saved_config.get("lbandclientparity_s", PARITY_NONE),
+            "rtscts_b": self.__app.saved_config.get("lbandclientrtscts_b", 0),
+            "xonxoff_b": self.__app.saved_config.get("lbandclientxonxoff_b", 0),
+            "timeout_f": self.__app.saved_config.get("lbandclienttimeout_f", 0.1),
+            "msgmode_n": self.__app.saved_config.get("lbandclientmsgmode_n", GET),
+            "userport_s": self.__app.saved_config.get("spartnport_s", ""),
+        }
         self._frm_spartn_serial = SerialConfigFrame(
+            self.__app,
             self,
             preselect=KNOWNGPS,
             timeouts=TIMEOUTS,
             bpsrates=BPSRATES,
             msgmodes=list(MSGMODES.keys()),
-            userport=userport,  # user-defined serial port
+            saved_config=saved_lband_config,
         )
         self._lbl_freq = Label(self, text="L-Band Frequency (Hz)")
         self._ent_freq = Entry(
@@ -237,6 +288,8 @@ class SPARTNLBANDDialog(Frame):
             width=10,
             wrap=True,
         )
+        self._lbl_ebno = Label(self, text="")
+        self._lbl_fec = Label(self, text="")
         self._btn_connect = Button(
             self,
             width=45,
@@ -291,31 +344,62 @@ class SPARTNLBANDDialog(Frame):
         self._ent_unqword.grid(column=1, row=9, columnspan=3, sticky=W)
         self._lbl_outport.grid(column=0, row=10, sticky=W)
         self._spn_outport.grid(column=1, row=10, columnspan=3, sticky=W)
+        self._lbl_ebno.grid(column=0, row=11, sticky=W)
+        self._lbl_fec.grid(column=1, row=11, sticky=W)
         ttk.Separator(self).grid(
-            column=0, row=11, columnspan=4, padx=2, pady=3, sticky=(W, E)
+            column=0, row=12, columnspan=4, padx=2, pady=3, sticky=(W, E)
         )
-        self._btn_connect.grid(column=0, row=12, padx=3, pady=2, sticky=W)
-        self._btn_disconnect.grid(column=1, row=12, padx=3, pady=2, sticky=W)
-        self._btn_send.grid(column=2, row=12, padx=3, pady=2, sticky=W)
-        self._lbl_send.grid(column=3, row=12, padx=3, pady=2, sticky=W)
+        self._btn_connect.grid(column=0, row=13, padx=3, pady=2, sticky=W)
+        self._btn_disconnect.grid(column=1, row=13, padx=3, pady=2, sticky=W)
+        self._btn_send.grid(column=2, row=13, padx=3, pady=2, sticky=W)
+        self._lbl_send.grid(column=3, row=13, padx=3, pady=2, sticky=W)
+
+    def _attach_events(self):
+        """
+        Bind events to frame.
+        """
+
+        self.__master.bind(SPARTN_ERR_EVENT, self.on_error)
 
     def _reset(self):
         """
         Reset configuration widgets.
         """
 
-        self._enabledbg.set(0)
         self._saveconfig.set(0)
-        self._spartn_drat.set(PMP_DATARATES["B2400"])
-        self._spartn_freq.set(SPARTN_DEFAULT["freq"])
-        self._spartn_schwin.set(SPARTN_DEFAULT["schwin"])
-        self._spartn_usesid.set(SPARTN_DEFAULT["usesid"])
-        self._spartn_sid.set(SPARTN_DEFAULT["sid"])
-        self._spartn_descrm.set(SPARTN_DEFAULT["descrm"])
-        self._spartn_prescrm.set(SPARTN_DEFAULT["prescrm"])
-        self._spartn_descrminit.set(SPARTN_DEFAULT["descrminit"])
-        self._spartn_unqword.set(SPARTN_DEFAULT["unqword"])
-        self._spartn_outport.set(PASSTHRU)
+        self._enabledbg.set(self.__app.saved_config.get("lbandclientdebug_b", 0))
+        self._spartn_drat.set(
+            self.__app.saved_config.get("lbandclientdrat_n", PMP_DATARATES["B2400"])
+        )
+        self._spartn_freq.set(
+            self.__app.saved_config.get("lbandclientfreq_n", D9S_CONFIG["freq"])
+        )
+        self._spartn_schwin.set(
+            self.__app.saved_config.get("lbandclientschwin_n", D9S_CONFIG["schwin"])
+        )
+        self._spartn_usesid.set(
+            self.__app.saved_config.get("lbandclientusesid_b", D9S_CONFIG["usesid"])
+        )
+        self._spartn_sid.set(
+            self.__app.saved_config.get("lbandclientsid_n", D9S_CONFIG["sid"])
+        )
+        self._spartn_descrm.set(
+            self.__app.saved_config.get("lbandclientdescrm_b", D9S_CONFIG["descrm"])
+        )
+        self._spartn_prescrm.set(
+            self.__app.saved_config.get("lbandclientprescrm_b", D9S_CONFIG["prescrm"])
+        )
+        self._spartn_descrminit.set(
+            self.__app.saved_config.get(
+                "lbandclientdescrminit_b", D9S_CONFIG["descrminit"]
+            )
+        )
+        self._spartn_unqword.set(
+            self.__app.saved_config.get("lbandclientunqword_s", D9S_CONFIG["unqword"])
+        )
+        self._spartn_outport.set(
+            self.__app.saved_config.get("lbandclientoutport_s", PASSTHRU)
+        )
         self.__container.set_status("")
         if self.__app.rtk_conn_status == CONNECTED_SPARTNLB:
             self.set_controls(CONNECTED_SPARTNLB)
@@ -329,6 +413,7 @@ class SPARTNLBANDDialog(Frame):
         :param int status: connection status (0 = disconnected, 1 = connected)
         """
 
+        self._frm_spartn_serial.set_status(status)
         stat = DISABLED if status == CONNECTED_SPARTNLB else NORMAL
         for wdg in (self._btn_connect,):
             wdg.config(state=stat)
@@ -382,6 +467,7 @@ class SPARTNLBANDDialog(Frame):
         Connect to Correction receiver.
         """
 
+        frm = self._frm_spartn_serial
         if self.__app.rtk_conn_status == CONNECTED_SPARTNIP:
             self.__container.set_status(
                 DLGSPARTNWARN.format("IP", "L-Band"),
@@ -389,16 +475,26 @@ class SPARTNLBANDDialog(Frame):
             )
             return
 
-        if self.serial_settings.status == NOPORTS:
+        if frm.status == NOPORTS:
             return
 
+        conndict = {
+            "read_event": SPARTN_EVENT,
+            "eof_event": SPARTN_EOF_EVENT,
+            "error_event": SPARTN_ERR_EVENT,
+            "inqueue": self.__app.spartn_inqueue,
+            "outqueue": self.__app.spartn_outqueue,
+            "socket_inqueue": self.__app.socket_inqueue,
+            "conntype": CONNECTED,
+            "msgmode": self.__app.frm_settings.frm_serial.msgmode,
+            "serial_settings": self._frm_spartn_serial,
+        }
+
         # start serial stream thread
         self.__app.rtk_conn_status = CONNECTED_SPARTNLB
-        self.__app.spartn_stream_handler.start_read_thread(self._get_settings())
+        self.__app.spartn_stream_handler.start_read_thread(self.__container, conndict)
         self.__container.set_status(
-            f"Connected to {self.serial_settings.port}:{self.serial_settings.port_desc} "
-            + f"@ {self.serial_settings.bpsrate}",
-            "green",
+            f"Connected to {frm.port}:{frm.port_desc} @ {frm.bpsrate}", "green"
         )
         self.set_controls(CONNECTED_SPARTNLB)
 
@@ -422,25 +518,15 @@ class SPARTNLBANDDialog(Frame):
                 )
             self.set_controls(DISCONNECTED)
 
-    def _get_settings(self) -> dict:
+    def on_error(self, event):  # pylint: disable=unused-argument
         """
-        Get settings dict.
+        EVENT TRIGGERED
+        Action on <<spartn_error>> event - disconnect.
 
-        :return: dictionary of settings for stream handler
-        :rtype: dict
+        :param event event: read event
         """
 
-        return {
-            "owner": self,
-            "read_event": SPARTN_EVENT,
-            "eof_event": SPARTN_EOF_EVENT,
-            "inqueue": self.__app.spartn_inqueue,
-            "outqueue": self.__app.spartn_outqueue,
-            "socket_inqueue": self.__app.socket_inqueue,
-            "socket_outqueue": self.__app.socket_outqueue,
-            "serial_settings": self.serial_settings,
-            "in_filepath": "",
-        }
+        self.on_disconnect()
 
     def _format_cfgpoll(self) -> UBXMessage:
         """
@@ -501,7 +587,9 @@ class SPARTNLBANDDialog(Frame):
             else:
                 cfgdata.append((f"CFG_INFMSG_UBX_{port}", b"\x07"))
         if outport in ("UART1", "UART2"):
-            cfgdata.append(f"CFG_{outport}_BAUDRATE", int(self.serial_settings.bpsrate))
+            cfgdata.append(
+                f"CFG_{outport}_BAUDRATE", int(self._frm_spartn_serial.bpsrate)
+            )
 
         msg = UBXMessage.config_set(SET_LAYER_RAM, TXN_NONE, cfgdata)
         return msg
@@ -596,64 +684,44 @@ class SPARTNLBANDDialog(Frame):
         elif msg.identity == "ACK-NAK":
             self._lbl_send.config(image=self._img_warn)
             self.__container.set_status(CONFIGBAD.format(CFGSET), "red")
+        elif msg.identity == "RXM-PMP":
+            self._lbl_ebno.config(text=f"EBNO: {msg.ebno}")
+            self._lbl_fec.config(text=f"FEC Bits: {msg.fecBits}")
+
         self.update_idletasks()
 
-    # ============================================
-    # FOLLOWING METHODS REQUIRED BY STREAM_HANDLER
-    # ============================================
-
     @property
-    def conn_status(self) -> int:
+    def settings(self) -> dict:
         """
-        Getter for connection status
-        (0 = disconnected, 1 = serial, 2 = socket, 4 = file).
+        Getter for settings.
+
+        :return: dictionary of settings
+        :rtype: dict
         """
 
-        return self.__app.rtk_conn_status
-
-    @conn_status.setter
-    def conn_status(self, status: int):
-        """
-        Setter for connection status.
-
-        :param int status: 0 = disconnected, 1 = serial, 2 = socket, 4 = file.
-        """
-
-        self.__app.rtk_conn_status = status
-
-    def set_status(self, msg: str, color: str = ""):
-        """
-        Set status message.
-
-        :param str message: message to be displayed
-        :param str color: rgb color of text (blue)
-        """
-
-        if color == "":
-            self.__container.set_status(msg)
-        else:
-            self.__container.set_status(msg, color)
-
-    def set_connection(self, msg: str, color: str = ""):
-        """
-        Set status message.
-
-        :param str message: message to be displayed
-        :param str color: rgb color of text (blue)
-        """
-
-        if color == "":
-            self.__container.set_status(msg)
-        else:
-            self.__container.set_status(msg, color)
-
-    @property
-    def serial_settings(self) -> Frame:
-        """
-        Getter for common serial configuration panel
-
-        :return: reference to serial form
-        :rtype: Frame
-        """
-
-        return self._frm_spartn_serial
+        try:
+            settings = {
+                "bpsrate": self._frm_spartn_serial.bpsrate,
+                "databits": self._frm_spartn_serial.databits,
+                "stopbits": self._frm_spartn_serial.stopbits,
+                "parity": self._frm_spartn_serial.parity,
+                "rtscts": self._frm_spartn_serial.rtscts,
+                "xonxoff": self._frm_spartn_serial.xonxoff,
+                "timeout": self._frm_spartn_serial.timeout,
+                "msgmode": self._frm_spartn_serial.msgmode,
+                "userport": self._frm_spartn_serial.userport,
+                "freq": int(self._spartn_freq.get()),
+                "schwin": int(self._spartn_schwin.get()),
+                "usesid": int(self._spartn_usesid.get()),
+                "sid": int(self._spartn_sid.get()),
+                "drat": int(self._spartn_drat.get()),
+                "descrm": int(self._spartn_descrm.get()),
+                "prescrm": int(self._spartn_prescrm.get()),
+                "descrminit": int(self._spartn_descrminit.get()),
+                "unqword": self._spartn_unqword.get(),
+                "outport": self._spartn_outport.get(),
+                "debug": int(self._enabledbg.get()),
+            }
+            return settings
+        except (TypeError, ValueError):
+            return {}
