@@ -1,13 +1,15 @@
 """
+spartn_mqtt_frame.py
+
 SPARTN configuration dialog
 
 This is the pop-up dialog containing the various
 SPARTN configuration functions.
 
-This dialog maintains its own threaded serial
-stream handler for incoming and outgoing Correction
-receiver (D9S) data (RXM-PMP), and must remain open for
-SPARTN passthrough to work.
+NB: The initial configuration for the SPARTN MQTT client
+(pygnssutils.GNSSMQTTClient) is set in app.update_SPARTN_handler().
+Once started, the persisted state for the SPARTN MQTT client is
+held in the threaded SPARTN handler itself, NOT in this frame.
 
 Created on 26 Jan 2023
 
@@ -51,13 +53,14 @@ from pygpsclient.globals import (
     ICON_SERIAL,
     ICON_SOCKET,
     ICON_WARNING,
-    OUTPORT_SPARTN,
     READONLY,
     RXMMSG,
+    SAVED_CONFIG,
     SPARTN_GNSS,
+    SPARTN_OUTPORT,
     SPARTN_PPREGIONS,
 )
-from pygpsclient.helpers import VALLEN, valid_entry
+from pygpsclient.helpers import MAXPORT, VALINT, VALLEN, valid_entry
 from pygpsclient.strings import DLGSPARTNWARN, LBLSPARTNIP, MQTTCONN
 
 
@@ -81,6 +84,7 @@ class SPARTNMQTTDialog(Frame):
         self.__app = app  # Reference to main application class
         self.__master = self.__app.appmaster  # Reference to root class (Tk)
         self.__container = container  # container frame
+        self._saved_config = kwargs.pop(SAVED_CONFIG, {})
 
         Frame.__init__(self, self.__container.container, *args, **kwargs)
 
@@ -98,7 +102,7 @@ class SPARTNMQTTDialog(Frame):
         self._status = StringVar()
         self._mqtt_server = StringVar()
         self._mqtt_port = IntVar()
-        self._mqtt_port.set(OUTPORT_SPARTN)
+        self._mqtt_port.set(SPARTN_OUTPORT)
         self._mqtt_region = StringVar()
         self._mqtt_clientid = StringVar()
         self._mqtt_crt = StringVar()
@@ -130,6 +134,14 @@ class SPARTNMQTTDialog(Frame):
             relief="sunken",
             width=25,
         )
+        self._lbl_mqttport = Label(self, text="Port")
+        self._ent_mqttport = Entry(
+            self,
+            textvariable=self._mqtt_port,
+            state=NORMAL,
+            relief="sunken",
+            width=8,
+        )
         self._lbl_mqttregion = Label(self, text="Region")
         self._spn_mqttregion = Spinbox(
             self,
@@ -215,33 +227,35 @@ class SPARTNMQTTDialog(Frame):
         self._ent_mqttserver.grid(
             column=1, row=1, columnspan=5, padx=3, pady=2, sticky=W
         )
-        self._lbl_mqttregion.grid(column=0, row=2, padx=3, pady=2, sticky=W)
+        self._lbl_mqttport.grid(column=0, row=2, padx=3, pady=2, sticky=W)
+        self._ent_mqttport.grid(column=1, row=2, padx=3, pady=2, sticky=W)
+        self._lbl_mqttregion.grid(column=0, row=3, padx=3, pady=2, sticky=W)
         self._spn_mqttregion.grid(
-            column=1, row=2, columnspan=5, padx=3, pady=2, sticky=W
-        )
-        self._lbl_mqttclientid.grid(column=0, row=3, padx=3, pady=2, sticky=W)
-        self._ent_mqttclientid.grid(
             column=1, row=3, columnspan=5, padx=3, pady=2, sticky=W
         )
-        self._lbl_topics.grid(column=0, row=4, padx=3, pady=2, sticky=W)
-        self._chk_mqtt_iptopic.grid(column=1, row=4, padx=3, pady=2, sticky=W)
-        self._chk_mqtt_mgatopic.grid(column=2, row=4, padx=3, pady=2, sticky=W)
-        self._chk_mqtt_keytopic.grid(column=3, row=4, padx=3, pady=2, sticky=W)
-        self._lbl_mqttcrt.grid(column=0, row=5, padx=3, columnspan=5, pady=2, sticky=W)
+        self._lbl_mqttclientid.grid(column=0, row=4, padx=3, pady=2, sticky=W)
+        self._ent_mqttclientid.grid(
+            column=1, row=4, columnspan=5, padx=3, pady=2, sticky=W
+        )
+        self._lbl_topics.grid(column=0, row=5, padx=4, pady=2, sticky=W)
+        self._chk_mqtt_iptopic.grid(column=1, row=5, padx=3, pady=2, sticky=W)
+        self._chk_mqtt_mgatopic.grid(column=2, row=5, padx=3, pady=2, sticky=W)
+        self._chk_mqtt_keytopic.grid(column=3, row=5, padx=3, pady=2, sticky=W)
+        self._lbl_mqttcrt.grid(column=0, row=6, padx=3, columnspan=5, pady=2, sticky=W)
         self._ent_mqttcrt.grid(
-            column=1, row=5, padx=3, columnspan=4, pady=2, sticky=(W, E)
+            column=1, row=6, padx=3, columnspan=4, pady=2, sticky=(W, E)
         )
-        self._btn_opencrt.grid(column=5, row=5, padx=3, pady=2, sticky=E)
-        self._lbl_mqttpem.grid(column=0, row=6, columnspan=3, padx=3, pady=2, sticky=W)
+        self._btn_opencrt.grid(column=5, row=6, padx=3, pady=2, sticky=E)
+        self._lbl_mqttpem.grid(column=0, row=7, columnspan=3, padx=3, pady=2, sticky=W)
         self._ent_mqttpem.grid(
-            column=1, row=6, columnspan=4, padx=3, pady=2, sticky=(W, E)
+            column=1, row=7, columnspan=4, padx=3, pady=2, sticky=(W, E)
         )
-        self._btn_openpem.grid(column=5, row=6, padx=3, pady=2, sticky=E)
+        self._btn_openpem.grid(column=5, row=7, padx=3, pady=2, sticky=E)
         ttk.Separator(self).grid(
-            column=0, row=7, columnspan=6, padx=2, pady=3, sticky=(W, E)
+            column=0, row=8, columnspan=6, padx=2, pady=3, sticky=(W, E)
         )
-        self._btn_connect.grid(column=0, row=8, padx=3, pady=2, sticky=W)
-        self._btn_disconnect.grid(column=2, row=8, padx=3, pady=2, sticky=W)
+        self._btn_connect.grid(column=0, row=9, padx=3, pady=2, sticky=W)
+        self._btn_disconnect.grid(column=2, row=9, padx=3, pady=2, sticky=W)
 
     def _reset(self):
         """
@@ -249,7 +263,8 @@ class SPARTNMQTTDialog(Frame):
         """
 
         self._get_settings()
-        self._mqtt_clientid.set(self.__app.app_config.get("mqttclientid", ""))
+        # self._mqtt_clientid.set(self._saved_config.get("mqttclientid_s", ""))
+        # self._mqtt_region.set(self._saved_config.get("mqttclientregion_s", "eu"))
         self._reset_keypaths(self._mqtt_clientid.get())
         self.__container.set_status(
             "",
@@ -318,6 +333,7 @@ class SPARTNMQTTDialog(Frame):
         stat = DISABLED if status == CONNECTED_SPARTNIP else NORMAL
         for wdg in (
             self._ent_mqttserver,
+            self._ent_mqttport,
             self._btn_connect,
             self._ent_mqttclientid,
             self._ent_mqttcrt,
@@ -343,6 +359,7 @@ class SPARTNMQTTDialog(Frame):
 
         valid = True
         valid = valid & valid_entry(self._ent_mqttserver, VALLEN, 1, 50)
+        valid = valid & valid_entry(self._ent_mqttport, VALINT, 1, MAXPORT)
         valid = valid & valid_entry(self._ent_mqttclientid, VALLEN, 1, 50)
         valid = valid & valid_entry(self._ent_mqttcrt, VALLEN, 1, 254)
         valid = valid & valid_entry(self._ent_mqttpem, VALLEN, 1, 254)
@@ -401,6 +418,8 @@ class SPARTNMQTTDialog(Frame):
                 "green",
             )
             self.__app.rtk_conn_status = CONNECTED_SPARTNIP
+        else:
+            self.__container.set_status("ERROR! Invalid Settings", "red")
 
     def on_disconnect(self, msg: str = ""):
         """
