@@ -1,9 +1,13 @@
 """
+ubx_config_dialog.py
+
 UBX configuration container dialog
 
 This is the pop-up dialog containing the various
 UBX configuration command frames.
 
+Supply initial settings via `config` keyword argument.
+
 NB: Individual UBX configuration commands do not have uniquely
 identifiable synchronous or asynchronous responses (e.g. unique
 txn ID). The way we keep tabs on confirmation status is to
@@ -26,10 +30,12 @@ from pyubx2 import UBXMessage
 
 from pygpsclient.globals import (
     CONNECTED,
+    CONNECTED_SOCKET,
     DLGTUBX,
     ENABLE_CFG_OTHER,
     ICON_EXIT,
     POPUP_TRANSIENT,
+    SAVED_CONFIG,
     UBX_CFGMSG,
     UBX_CFGOTHER,
     UBX_CFGPRT,
@@ -66,6 +72,8 @@ class UBXConfigDialog(Toplevel):
 
         self.__app = app  # Reference to main application class
         self.__master = self.__app.appmaster  # Reference to root class (Tk)
+        self._saved_config = kwargs.pop(SAVED_CONFIG, {})
+
         Toplevel.__init__(self, app)
         if POPUP_TRANSIENT:
             self.transient(self.__app)
@@ -125,7 +133,14 @@ class UBXConfigDialog(Toplevel):
             self.__app, self, borderwidth=2, relief="groove"
         )
         self._frm_preset = UBX_PRESET_Frame(
-            self.__app, self, borderwidth=2, relief="groove"
+            self.__app,
+            self,
+            borderwidth=2,
+            relief="groove",
+            # cater for old and new config file element names...
+            saved_config=self._saved_config.get(
+                "ubxpresets_l", self._saved_config.get("ubxpresets", [])
+            ),
         )
 
     def _do_layout(self):
@@ -222,7 +237,7 @@ class UBXConfigDialog(Toplevel):
         self._frm_config_port.reset()
         self._frm_config_dynamic.reset()
         self._frm_device_info.reset()
-        if self.__app.conn_status != CONNECTED:
+        if self.__app.conn_status not in (CONNECTED, CONNECTED_SOCKET):
             self.set_status("Device not connected", "red")
 
     def set_pending(self, msgid: int, ubxfrm: int):
@@ -333,10 +348,16 @@ class UBXConfigDialog(Toplevel):
 
     def send_command(self, msg: UBXMessage):
         """
-        Send command to receiver and record to memory
-        if in 'record' mode.
+        Send command to receiver.
         """
 
         self.__app.gnss_outqueue.put(msg.serialize())
+        self.record_command(msg)
+
+    def record_command(self, msg: UBXMessage):
+        """
+        Record command to memory if in 'record' mode.
+        """
+
         if self.recordmode:
             self._frm_recorder.update_record(msg)
