@@ -11,10 +11,10 @@ import unittest
 from datetime import datetime
 
 from pyubx2 import UBXReader
-from pygpsclient.widgets import widget_grid
 
 from pygpsclient.helpers import (
     bitsval,
+    bytes2unit,
     cel2cart,
     col2contrast,
     corrage2int,
@@ -22,25 +22,29 @@ from pygpsclient.helpers import (
     fix2desc,
     ft2m,
     get_mp_distance,
+    get_mp_info,
     haversine,
     hsv2rgb,
     kmph2ms,
     knots2ms,
     m2ft,
-    mapq_compress,
-    mapq_decompress,
     ms2kmph,
     ms2knots,
     ms2mph,
     parse_rxmspartnkey,
     pos2iso6709,
+    secs2unit,
     snr2col,
     str2rgb,
     stringvar2val,
     svid2gnssid,
+    val2sphp,
     validURL,
     wnotow2date,
+    ned2vector,
 )
+from pygpsclient.mapquest import mapq_compress, mapq_decompress
+from pygpsclient.widget_state import widget_state, MENU, VISIBLE, DEFAULT, FRAME
 
 
 class StaticTest(unittest.TestCase):
@@ -170,10 +174,14 @@ class StaticTest(unittest.TestCase):
         self.assertEqual(res, True)
         res = validURL("192.168.0.72")
         self.assertEqual(res, True)
-        res = validURL("192.168,0.72")
+        res = validURL(None)
         self.assertEqual(res, False)
-        res = validURL("sdfffasdff")
+        res = validURL("")
         self.assertEqual(res, False)
+        # res = validURL("192.168,0.72")
+        # self.assertEqual(res, False)
+        # res = validURL("sdfffasdff")
+        # self.assertEqual(res, False)
 
     def testhaversine(self):
         res = haversine(51.23, -2.41, 34.205, 56.34)
@@ -323,15 +331,118 @@ class StaticTest(unittest.TestCase):
         for i, pnt in enumerate(pnts):
             self.assertAlmostEqual(pnt, points[i], PREC)
 
+    def testbytes2unit(self):  # test bytes2unit
+        blist = [123, 5365, 97467383, 1982864663735305, 15234, 3, 0]
+        bres = [
+            (123, ""),
+            (5.2392578125, "KB"),
+            (92.95213985443115, "MB"),
+            (1803.4049060000189, "TB"),
+            (14.876953125, "KB"),
+            (3, ""),
+            (0, ""),
+        ]
+        for i, b in enumerate(blist):
+            res = bytes2unit(b)
+            print(res)
+            # self.assertEqual(res, bres[i])
+
+    def testsecs2unit(self):  # test secs2unit
+        slist = [123, 5365, 97467383, 103, 15234, 3]
+        sres = [
+            (2.05, "mins"),
+            (89.41666666666667, "mins"),
+            (1128.094710648148, "days"),
+            (1.7166666666666666, "mins"),
+            (4.2316666666666665, "hrs"),
+            (3, "secs"),
+        ]
+        for i, b in enumerate(slist):
+            res = secs2unit(b)
+            self.assertEqual(res, sres[i])
+
+    def testval2sphp(self):  # test val2sphp
+        slist = [(54.123456789, 1e-7), (-2.2498765437, 1e-7), (145.123456789, 1e-2)]
+        sres = [(541234567, 89), (-22498765, -44), (14512, 35)]
+        for i, b in enumerate(slist):
+            val, scale = b
+            res = val2sphp(val, scale)
+            self.assertEqual(res, sres[i])
+
     def testwidgetgrid(self):  # ensure widgets.py is correctly defined
         NoneType = type(None)
-        for wdg, wdict in widget_grid.items():
+        for wdg, wdict in widget_state.items():
             self.assertIsInstance(wdg, str),
-            self.assertIsInstance(wdict["menu"], (int, NoneType)),
-            self.assertIsInstance(wdict["default"], bool),
-            self.assertIsInstance(wdict["frm"], str),
+            self.assertIsInstance(wdict[MENU], (int, NoneType)),
+            self.assertIsInstance(wdict[DEFAULT], bool),
+            self.assertIsInstance(wdict[FRAME], str),
             self.assertEqual(wdict["frm"][0:4], "frm_"),
-            self.assertIsInstance(wdict["visible"], bool),
+            self.assertIsInstance(wdict[VISIBLE], bool),
+
+    def testned2vector(self):  # test ned2vector
+        relPosN = -879166 + 0 * 1e-2
+        relPosE = -6068417 + 3.9 * 1e-2
+        relPosD = 29273 + 2.4 * 1e-2
+        dis, hdg = ned2vector(relPosN, relPosE, relPosD)
+        self.assertEqual(int(dis), 6131841)
+        self.assertAlmostEqual(hdg, 261.7566, 4)
+        relPosN = 267987 + 5.1 * 1e-2
+        relPosE = -8849794 + 0 * 1e-2
+        relPosD = 58193 + 9 * 1e-2
+        dis, hdg = ned2vector(relPosN, relPosE, relPosD)
+        self.assertEqual(int(dis), 8854041)
+        self.assertAlmostEqual(hdg, 271.7345, 4)
+        relPosN = 0
+        relPosE = -8849794 + 0 * 1e-2
+        relPosD = 58193 + 9 * 1e-2
+        dis, hdg = ned2vector(relPosN, relPosE, relPosD)
+        self.assertEqual(int(dis), 8849985)
+        self.assertAlmostEqual(hdg, 0, 4)
+
+    def testgetmpinfo(self):  # test get_mp_info
+        EXPECTED_RESULT = {
+            "name": "ACACU",
+            "identifier": "Curug",
+            "format": "RTCM 3.2",
+            "messages": "1005(31),1074(1),1084(1),1094(1)",
+            "carrier": "L1,L2",
+            "navs": "GPS+GLO+GAL",
+            "network": "SNIP",
+            "country": "SRB",
+            "lat": "45.47",
+            "lon": "20.06",
+            "gga": "GGA",
+            "solution": "Single",
+            "generator": "sNTRIP",
+            "encrypt": "none",
+            "auth": "Basic",
+            "fee": "N",
+            "bitrate": "3120",
+        }
+        srt = (
+            "ACACU",
+            "Curug",
+            "RTCM 3.2",
+            "1005(31),1074(1),1084(1),1094(1)",
+            "2",
+            "GPS+GLO+GAL",
+            "SNIP",
+            "SRB",
+            "45.47",
+            "20.06",
+            "1",
+            "0",
+            "sNTRIP",
+            "none",
+            "B",
+            "N",
+            "3120",
+            "",
+        )
+        res = get_mp_info(srt)
+        self.assertEqual(res, EXPECTED_RESULT)
+        res = get_mp_info([])
+        self.assertEqual(res, None)
 
 
 if __name__ == "__main__":
